<?php

namespace Acme\SosMonBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Acme\SosMonBundle\Form\Type\AlertsystemType;
use Acme\SosMonBundle\Model\Resanswer;
use Acme\SosMonBundle\Model\Common;

class AlertsystemsController extends Controller
{
	
/**
* list of systems
* /data
* @return json array of categories
*/
    public function dataAction($opt)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$access = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'contacts');

		if ($access == 0) { // guest cannot see any users
			return new Response(json_encode(array()));
		}
		else {
			$list = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:Alertsystem')->getList( $usr->getOwnid(), $opt );	
			return new Response(json_encode($list));
		}
    }
	
/**
* create or edit station
* /entity
* @param int $id
* @return view
*/
	public function entityAction($id, Request $request)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$access = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'stations');

		if ($access == 0) { // no access 
			return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
				'message'  => 'User '. $usr->getName().' has no access to stations information',
			));		
		}
		
		$entity = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:Alertsystem')->findEntity($id, $usr->getOwnid() );
		$entity['access'] = $access;
		
		$form = $this->createForm(new AlertsystemType(), $entity);
				
		return $this->render('AcmeSosMonBundle:Alertsystem:alertsystem.html.twig', array(
			'form'   => $form->createView(), 
			'entity' => $entity,
		));
    }
	
/**
* save Station
* /save
* @param int $id
* @return json array('id','message','success')
*/
	public function saveAction(Request $request)
	{
		if ($request->getMethod() == 'POST') {
		
			$em = $this->getDoctrine()->getManager();
			$usr= $this->get('security.context')->getToken()->getUser();
			$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'stations');

			if ($access == 0) { // no access 
				return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
					'message'  => 'User '. $usr->getName().' can not save station information',
				));		
			}
		
			$var = $request->request->all();	
			$params = $var['alertsystem'];

			$id = $em->getRepository('AcmeSosMonBundle:Alertsystem')->saveEntity( $params );
			$em->getRepository('AcmeSosMonBundle:Alertcontacts')->assignContacts( $id, explode( ',', $params['contacts'] ) );
			
			$this->get('call_actlog')->regActlog( $usr->getOwnid(), $usr->getId(),
												 'location "'.$params['name'].'"'.((int)$params['id'] == 0 ? ' created' :' modified'),
												 'Locations', $id, 0);
					
			return new Response(Resanswer::getRetJSON('',$id > 0, $id));	
		}	
	}	

/**
* contacts for system
* /systems_conracts
* @param int $id
* @return array
*/
	public function contactsAction(Request $request)
    {	
		$usr= $this->get('security.context')->getToken()->getUser();
		$var = $request->request->all();
		
		$list = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:Alertcontacts')->getContacts( $var['id'], $usr->getOwnid() );
		
		return new Response(json_encode($list));
    }
	
/**
* delete Stations 	
* /delete
* @param int $id
* @return json array('id','message','success')
	*/	
	public function deleteAction($id)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		
		$access = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'stations');
		if ($access == 0) { // no access 
			return new Response(Resanswer::getRetJSON('User '. $usr->getName().' can not delete station',false, $id));
		}	
		
		$count = $em->getRepository('AcmeSosMonBundle:Alertstations')->countStations($id);
		if ($count == 0) {
			$entity = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:Alertsystem')->deleteEntity($id );	
			
			if ($entity) {
				$this->get('call_actlog')->regActlog($usr->getOwnid(), $usr->getId(),'Location "'.$entity->getName().'" deleted', 'LOCATION', $id, 0);					
			}
		}
		return new Response(Resanswer::getRetJSON('Alert system has got stations and can not be deleted',($count == 0), $count));
    }	

}
