<?php

namespace Acme\SosMonBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Acme\SosMonBundle\Form\Type\ContactType;
use Acme\SosMonBundle\Model\Resanswer;

class ContactsController extends Controller
{
	
/**
* contacts main screen: list of available contacts
* /index
* @return view
*/
    public function indexAction()
    {
		$usr= $this->get('security.context')->getToken()->getUser();	
		$access = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'contacts');
		
		if ($access == 0) { // no access 
			return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
				'message'  => 'User '. $usr->getName().' has no access to contacts information',
			));		
		}
		else {
			return $this->render('AcmeSosMonBundle:Contact:contacts.html.twig', 
				array('access' => $access)
			);
		}
    }
	
/**
* list of contacts as array	
* /data
* @return json array of categories
*/
    public function dataAction($opt)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'contacts');

		if ($access == 0) { // guest cannot see any users
			return new Response(json_encode(array()));
		}
		else {
			return new Response(json_encode(
				$em->getRepository('AcmeSosMonBundle:Contact')->getList( $usr->getOwnid(), $opt )
			));
		}
    }	
	
/**
* create or edit contact
* /entity
* @param int $id
* @return view
*/
	public function entityAction($id, Request $request)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'contacts');

		if ($access == 0) { // no access 
			return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
				'message'  => 'User '. $usr->getName().' has no access to contacts information',
			));		
		}
		
		$entity = $em->getRepository('AcmeSosMonBundle:Contact')->findEntity($id, $usr->getOwnid());
		$entity['access'] = $access;
		
		$form = $this->createForm(new ContactType(), $entity);
				
		return $this->render('AcmeSosMonBundle:Contact:contact.html.twig', array(
			'form'   => $form->createView(), 
			'entity' => $entity,
		));
    }
	
/**
* contact as array
* /id
* @param int $id
* @return view
*/
	public function systemsAction( Request $request )
    {
		$usr = $this->get('security.context')->getToken()->getUser();
		$var = $request->request->all();
        //$ownid = isset($var['ownid']) ?(int) $var['ownid'] : $usr->getOwnid();
		$id  = isset( $var['id'] ) ? (int) $var['id'] : 0 ;
			
		return new Response(json_encode(
			$this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:Alertcontacts')->getAlerts($usr->getOwnid(), $id)
		));
    }
	
/**
* contact as array
* /id
* @param int $id
* @return view
*/
	public function idAction($id)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'contacts');

		if ($access == 0) { // no access 
			return new Response(json_encode( array() ));	
		}
	
		return new Response(json_encode(
					$em->getRepository('AcmeSosMonBundle:Contact')->findEntity($id, $usr->getOwnid())
				));
    }

/**
* save Contact
* /save
* @param int $id
* @return json array('id','message','success')
*/
	public function saveAction(Request $request)
	{
		if ($request->getMethod() == 'POST') {

			$usr= $this->get('security.context')->getToken()->getUser();
			$em = $this->getDoctrine()->getManager();
			$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'contacts');

			if ($access == 0) { // no access 
				return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
					'message'  => 'User '. $usr->getName().' can not save contact information',
				));		
			}
			$message = '';
			
			$var = $request->request->all();	
			$params = $var['contact'];
			
			if( $em->getRepository('AcmeSosMonBundle:Contact')->checkPhone( $params ) ) {
						
				$id = $em->getRepository('AcmeSosMonBundle:Contact')->saveEntity( $params);
				$em->getRepository('AcmeSosMonBundle:Alertcontacts')->assignSystems( $id, explode( ',', $params['systems'] ) );
			
				$this->get('call_actlog')->regActlog( $usr->getOwnid(), $usr->getId(),
											 'Contact "'.$params['fname'].' '.$params['lname'].'"'.((int)$params['ownid'] == 0 ? ' created' :' modified'),
											 'CONTACTS', $id, 0);								
			}
			else {
				$id = 0;
				$message = 'Phone is not unique : same number belongs to other person';
			}
					
			return new Response(Resanswer::getRetJSON($message, $id > 0, $id));	
		}	
	}	
	
/**
* delete Contact (set deleted = 1)	
* /delete
* @param int $id
* @return json array('id','message','success')
	*/	
	public function deleteAction($id)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'contacts');
		if ($access == 0) { // no access 
			return new Response(Resanswer::getRetJSON('User '. $usr->getName().' can not delete contact',false, $id));
		}	
		
		$entity = $em->getRepository('AcmeSosMonBundle:Contact')->deleteEntity($id );	

		if ($entity) {
			$em->getRepository('AcmeSosMonBundle:Alertcontacts')->deleteForContact( $id );
			
			$this->get('call_actlog')->regActlog($usr->getOwnid(), $usr->getId(),'Customer "'.$entity->getLname().' '.$entity->getFname().'" deleted', 'CUSTOMERS', $id, 0);		
		}		
				
		return new Response(Resanswer::getRetJSON('',true, $id));
    }	

}
