<?php

namespace Acme\SosMonBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Acme\SosMonBundle\Form\Type\CustomerType;
use Acme\SosMonBundle\Model\Resanswer;

class CustomersController extends Controller
{
	
/*
* customers main screen: list of available customers
* /index
* @return view
*/
    public function indexAction()
    {
		return $this->render('AcmeSosMonBundle:Customer:customers.html.twig');
    }
	
/**
* list of customers as array	
* /data
* @return json array of categories
*/
    public function dataAction()
    {	
		return new Response(json_encode(
				$this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:Customer')->getList()
				));	
    }	
	
/**
* list of customers as array	
* /combodata/opt
* @return json array of categories
*/
    public function combodataAction($opt)
    {
		return new Response(json_encode(
				$this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:Customer')->getList($opt)
				));
    }	
		
/**
* edit customer. Both admin and customer
* /settings
* @param int $id
* @return view
*/
	public function settingsAction($id, Request $request)
    {
		// access check is set for customer`s user
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'users');

		if ($access == 0) { // no access 
			return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
				'message'  => 'User '. $usr->getName().' has no access to customers information',
			));		
		}
		
		$entity = $em->getRepository('AcmeSosMonBundle:Customer')->findEntity($id);
		$entity['access']    = $access;
		$entity['markerimg'] = array(1 => 'Standard', 
									 2 => 'Points');
		
		$form = $this->createForm(new CustomerType(), $entity);
		
		$helper = $this->container->get('templating.helper.assets');
				
		return $this->render('AcmeSosMonBundle:Customer:settings.html.twig', array(
			'form'  => $form->createView(), 
			'entity'=> $entity,
			'icon1' => $helper->getUrl('bundles/acmesosmon/images/icon'.$entity['mstype'].'1.png'),
			'icon2' => $helper->getUrl('bundles/acmesosmon/images/icon'.$entity['mstype'].'2.png'),
			'icon3' => $helper->getUrl('bundles/acmesosmon/images/icon'.$entity['mstype'].'3.png'),
		));
    }

/**
* save Customer
* /save
* @param int $id
* @return json array('id','message','success')
*/
	public function saveAction(Request $request)
	{
		if ($request->getMethod() == 'POST') {
			// access check is set for customer`s user
			$usr= $this->get('security.context')->getToken()->getUser();
			$em = $this->getDoctrine()->getManager();
			$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'users');

			if ($access == 0) { // no access 
				return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
					'message'  => 'User '. $usr->getName().' can not save customers information',
				));		
			}
		
			$var = $request->request->all();	
			$params = $var['customer'];
			$id = $em->getRepository('AcmeSosMonBundle:Customer')->saveEntity( $params );
			
			$this->get('call_actlog')->regActlog( $usr->getOwnid(), $usr->getId(),
												 'Customer "'.$params['name'].'"'.((int)$params['ownid'] == 0 ? ' created' :' modified'),
												 'CUSTOMERS', $id, 0);
						
			return new Response(Resanswer::getRetJSON('', $id > 0, $id));	
		}	
	}


/**
* active Customer
* /active
* @param int $id
* @return json array('id','message','success')
*/	
	public function activeAction($id)
    {		
		return new Response(
			Resanswer::getRetJSON('',true, $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:Customer')->activeEntity( $id ) )
			);
    }	

/**
* get Icon images
* /markerimg
* @param int $id
* @return array
	*/	
	public function markerimgAction( $id )
    {
		$helper = $this->container->get('templating.helper.assets');
		return new Response(json_encode(array(
								'icon1' => $helper->getUrl('bundles/acmesosmon/images/icon'.$id.'1.png'),
								'icon2' => $helper->getUrl('bundles/acmesosmon/images/icon'.$id.'2.png'),
								'icon3' => $helper->getUrl('bundles/acmesosmon/images/icon'.$id.'3.png'))
		));
    }
	
/**
* check Customer prefix
* /delete
* @param int $id
* @return json array('id','message','success')
	*/	
	public function checkprefixAction( $prefix )
    {
	    $rep = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:Customer');
		
		$id      = 0;
		$flag    = false;
		
		if ( $rep->checkPrefix( $prefix ) ) {
		// create new customer. The only place where it happens is here 
			$entity = $rep->findEntity( 0 );
			$entity['prefix']  = $prefix;
			$entity['mstype']  = 1;   // marker type
			$entity['deflat']  = 45;  // default GPS position
			$entity['deflong'] = -90; // 
			$entity['active']  = 1;   // set as active
			
			$id = $rep->saveEntity( $entity );
			$flag = true;
		}
		
		return new Response(Resanswer::getRetJSON('', $flag, $id));
    }
	
/**
* delete Customer (set deleted = 1)	
* /delete
* @param int $id
* @return json array('id','message','success')
	*/	
	public function deleteAction($id)
    {
		$usr= $this->get('security.context')->getToken()->getUser();		
		$entity = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:Customer')->deleteEntity( $id );	
		
		if ($entity) {
			$this->get('call_actlog')->regActlog($usr->getOwnid(), $usr->getId(),'Customer "'.$entity->getName().'" deleted', 'CUSTOMERS', $id, 0);		
		}	
		return new Response(Resanswer::getRetJSON('',true, $id));
    }	

}
