<?php

namespace Acme\SosMonBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Acme\SosMonBundle\Model\Resanswer;
use Acme\SosMonBundle\Model\Common;
use Acme\SosMonBundle\Model\ExcelProcessing;

class HistorylogController extends Controller
{
	
/**
* events main screen: list of available events
* /index
* @return view
*/
    public function indexAction($ownid)
    {
		$usr= $this->get('security.context')->getToken()->getUser();	
		$access = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'events');
		
		if ($access == 0) { // no access 
			return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
				'message'  => 'User '. $usr->getName().' has no access to events information',
			));		
		}
		else {
			return $this->render('AcmeSosMonBundle:Historylog:historylogs.html.twig', 
				array('access' => $access,
					  'date_from' => date("m/d/Y"),
					  'date_to'   => date("m/d/Y"),
					  'ownid'     => $ownid,
					  )
			);
		}
    }
	
/**
* list of events as array	
* /data
* @return json array 
*/
    public function dataAction( Request $request)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
	
		$access = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'events');

		if ($access == 0) { // guest cannot see any users
			return new Response(json_encode(array()));
		}
		else {
	
			$var = $request->request->all();	
			
			$ownid       = isset($var['ownid'])? (int)$var['ownid'] : (int)$usr->getOwnid();
			$userid      = isset($var['userid'])? (int)$var['userid'] : 0;
			$dt1         = isset($var['dt1'])? Common::zfdate($var['dt1'], 'us','mysql') : Common::zfdate(date("m/d/Y", strtotime( "-1 month" )), 'us','mysql');
			$dt2         = isset($var['dt2'])? Common::zfdate($var['dt2'], 'us','mysql') : Common::zfdate(date("m/d/Y"), 'us','mysql');
			$rows        = isset($var['rows']) ?(int)$var['rows']:0;
			if (isset($var['firstpage'])) {
				$offset = 0;
			} 
			else {
				$offset      = isset($var['page']) ?((int)$var['page'] - 1) * $rows:0;
			}
					
			$list = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:Historylog')->getList( $rows, $offset, $userid, $ownid,  $dt1, $dt2);	
			return new Response(json_encode($list));	
			
		}
    }	
	
/**
* historylog excel file	
* /historylog
* @return request output
*/
    public function historylogexportAction( $target, $ownid, $userid, $dt1, $dt2, Request $request)
    {

		$dt1         = isset($dt1)? Common::zfdate($dt1, 'us','mysql') : Common::zfdate(date("m/d/Y"), 'us','mysql');
		$dt2         = isset($dt2)? Common::zfdate($dt2, 'us','mysql') : Common::zfdate(date("m/d/Y"), 'us','mysql');
		
		// export starts	
		$excel = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:Historylog')->exportHistorylog( 
						 $this->get('phpexcel')->createPHPExcelObject(),
						 array( 'ownid'  => $ownid, 
						        'userid' => $userid, 
								'dt1'    => $dt1,
								'dt2'    => $dt2)
						);
						
		$response = new Response();
		if ($target == 'excel') {
			$response = $this->get('ExcelProcessing')->excelOut( $excel, 'history_log');	
		}
		if ($target == 'pdf') {
			$response = $this->get('ExcelProcessing')->pdfOut( $excel, 'history_log');	
		}					

		return $response; 		
	}		

}
