<?php

namespace Acme\SosMonBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Acme\SosMonBundle\Form\Type\IbuttonType;
use Acme\SosMonBundle\Model\Resanswer;

class IbuttonsController extends Controller
{
	
/**
* ibuttons main screen: list of available ibuttons
* /index
* @return view
*/
    public function indexAction()
    {
		$usr= $this->get('security.context')->getToken()->getUser();	
		$access = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'ibuttons');
		
		if ($access == 0) { // no access 
			return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
				'message'  => 'User '. $usr->getName().' has no access to ibuttons information',
			));		
		}
		else {
			return $this->render('AcmeSosMonBundle:Ibutton:ibuttons.html.twig', 
				array('access' => $access)
			);
		}
    }
	
/**
* list of ibuttons as array	
* /data
* @return json array 
*/
    public function dataAction($opt, Request $request)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'ibuttons');

		if ($access == 0) { // guest cannot see any users
			return new Response(json_encode(array()));
		}
		else {
		
			$var = $request->request->all();	
			return new Response(json_encode(
					$em->getRepository('AcmeSosMonBundle:Ibutton')->getList( $usr->getOwnid(), $opt, (isset($var['key'])?$var['key']:'') )
				));
		}
    }	

/**
* excel file	
* /export
* @return request output
*/
    public function exportAction($target, Request $request)
    {
		$usr= $this->get('security.context')->getToken()->getUser();	
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'ibuttons');

		if ($access == 0) { // guest cannot see any users
			return new Response('');
		}
		else {	
		// export starts			
			$excel = $em->getRepository('AcmeSosMonBundle:Ibutton')->exportList( 
				$this->get('phpexcel')->createPHPExcelObject(),
				$usr->getOwnid()
			);	

			$response = new Response();
			if ($target == 'excel') {
				$response = $this->get('ExcelProcessing')->excelOut( $excel, 'ibuttons_list');	
			}
			if ($target == 'pdf') {
				$response = $this->get('ExcelProcessing')->pdfOut( $excel, 'ibuttons_list');	
			}					
		}
	}
	
/**
* create or edit ibutton
* /entity
* @param int $id
* @return view
*/
	public function entityAction($id, Request $request)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'ibuttons');
		
		if ($access == 0) { // no access 
			return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
				'message'  => 'User '. $usr->getName().' has no access to ibuttons information',
			));		
		}
		
		$entity = $em->getRepository('AcmeSosMonBundle:Ibutton')->findEntity($id, $usr->getOwnid());
		$entity['access'] = $access;
		$entity['free_keyholders'] = $em->getRepository('AcmeSosMonBundle:Keyholder')->getAvailableKeyholders($usr->getOwnid(), $entity['id']);
		$entity['keyholderid'] = $em->getRepository('AcmeSosMonBundle:Keyholder')->getIbuttonOwner($entity['id']);
		
		$form = $this->createForm(new IbuttonType(), $entity);
				
		return $this->render('AcmeSosMonBundle:Ibutton:ibutton.html.twig', array(
			'form'   => $form->createView(), 
			'entity' => $entity,
		));
    }

/**
* list of keystations as array	
* /keystations
* @return json array 
*/
    public function keysAction( Request $request)
    {	
		$var = $request->request->all();
		$em = $this->getDoctrine()->getManager();
	
		$key     = isset($var['ibuttonid'])? (int)$var['ibuttonid'] : 0;
		$station = isset($var['stationid'])? (int)$var['stationid'] : 0;

		return new Response(json_encode(
				$em->getRepository('AcmeSosMonBundle:Keystations')->getKeystations($key, $station)
			));
    }
	
/**
* save Ibutton
* /save
* @param int $id
* @return json array('id','message','success')
*/
	public function saveAction(Request $request)
	{
		if ($request->getMethod() == 'POST') {

			$em = $this->getDoctrine()->getManager();
			$usr= $this->get('security.context')->getToken()->getUser();
			$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'ibuttons');

			if ($access == 0) { // no access 
				return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
					'message'  => 'User '. $usr->getName().' can not save ibutton information',
				));		
			}

			$var = $request->request->all();
			$params = $var['ibutton'];
			
			$check = $em->getRepository('AcmeSosMonBundle:Ibutton')->checkEntity( $params );
			if ($check['success'] == true) {
				$id = $em->getRepository('AcmeSosMonBundle:Ibutton')->saveEntity( $params);
				$em->getRepository('AcmeSosMonBundle:Keystations')->assignStations( $id, explode( ',', $params['keystations'] ) );
			
				$this->get('call_actlog')->regActlog($usr->getOwnid(), $usr->getId(),'Key "'.$params['key'].'"'.((int)$params['id'] == 0 ? ' created' :' modified'), 'IBUTTON', $id, 0);		
						
				return new Response(Resanswer::getRetJSON('',$id > 0, $id));	
			}
			else {
				return new Response(Resanswer::getRetJSON($check['message'],false, 0));				
			}
		}	
	}	
/**
* active Ibuttons 	
* /delete
* @param int $id
* @return json array('id','message','success')
	*/	
	public function activeAction($id)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'ibuttons');
		if ($access == 0) { // no access 
			return new Response(Resanswer::getRetJSON('User '. $usr->getName().' can not delete ibutton',false, $id));
		}	
		
		$em->getRepository('AcmeSosMonBundle:Ibutton')->activeEntity($id);			
		return new Response(Resanswer::getRetJSON('',true, $id));
    }		
	
/**
* delete Ibuttons 	
* /delete
* @param int $id
* @return json array('id','message','success')
	*/	
	public function deleteAction($id)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'ibuttons');
		if ($access == 0) { // no access 
			return new Response(Resanswer::getRetJSON('User '. $usr->getName().' can not delete ibutton',false, $id));
		}		
		
		$entity = $em->getRepository('AcmeSosMonBundle:Ibutton')->deleteEntity($id );
		
		if ($entity) {
			$em->getRepository('AcmeSosMonBundle:Keyholder')->freeKey($id );		
			$this->get('call_actlog')->regActlog($usr->getOwnid(), $usr->getId(),'Key "'.$entity->getKey().'" deleted', 'IBUTTON', $id, 0);		
		}		
				
		return new Response(Resanswer::getRetJSON('',true, $id));
    }	
	
/**
* itransfer screen
* /itransfer
* @return view
*/
    public function itransferAction()
    {
		$usr= $this->get('security.context')->getToken()->getUser();	
		$access = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'ibuttons');
		
		if ($access == 0) { // no access 
			return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
				'message'  => 'User '. $usr->getName().' has no access to ibuttons information',
			));		
		}
		else {
			return $this->render('AcmeSosMonBundle:Ibutton:itransfer.html.twig', 
				array('access' => $access)
			);
		}
    }	
	
/**
* itransfer screen
* /itransfer
* @return view
*/
    public function dotransferAction($key, $person)
    {
		$usr= $this->get('security.context')->getToken()->getUser();	
		$em = $this->getDoctrine()->getManager();
		$access = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'ibuttons');
		
		if ($access == 0) { // no access 
			return new Response(Resanswer::getRetJSON('',false, 0));		
		}
		else {
		
			$em->getRepository('AcmeSosMonBundle:Keyholder')->assignButton( $person, $key );				
			return new Response(Resanswer::getRetJSON('',true, 0));
		}
    }	
	
	
/**
* itransfer screen
* /itransfer
* @return view
*/
    public function iassignAction()
    {
		$usr= $this->get('security.context')->getToken()->getUser();	
		$access = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'ibuttons');
		
		if ($access == 0) { // no access 
			return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
				'message'  => 'User '. $usr->getName().' has no access to ibuttons information',
			));		
		}
		else {
			return $this->render('AcmeSosMonBundle:Ibutton:iassign.html.twig', 
				array('access' => $access)
			);
		}
    }
	
/**
* itransfer screen
* /itransfer
* @return view
*/
    public function doassignAction($key, $loc)
    {
		$usr= $this->get('security.context')->getToken()->getUser();	
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'ibuttons');
		
		if ($access == 0) { // no access 
			return new Response(Resanswer::getRetJSON('User has no access to this feature.',false, 0));		
		}
		else {
			$count = $em->getRepository('AcmeSosMonBundle:Alertstations')->assignKeyForLocation( $key, $loc);
			return new Response(Resanswer::getRetJSON('Assignment successfully completed. Assigned to '.$count.' stations',true, 0));
		}
    }	
	
/**
* translist
* /translist
* @return view
*/
    public function translistAction()
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'ibuttons');

		if ($access == 0) { // guest cannot see any buttons
			return new Response(json_encode(array()));
		}
		else {
			$list = $em->getRepository('AcmeSosMonBundle:Ibutton')->getList( $usr->getOwnid(), '2' );
			$res = array();
			foreach($list as $row) {
				$res[] = array('id' => $row['id'],
							   'key' => $row['key'].(($row['keyholderid'] > 0 ) ? ' ('.$row['name'].')' : '')
							   );
			}		
			
			return new Response(json_encode($res));
		}
    }	

}
