<?php

namespace Acme\SosMonBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Acme\SosMonBundle\Form\Type\KeyholderType;
use Acme\SosMonBundle\Model\Resanswer;

class KeyholdersController extends Controller
{
	
/**
* keyholders main screen: list of available keyholders
* /index
* @return view
*/
    public function indexAction()
    {
		$usr= $this->get('security.context')->getToken()->getUser();	
		$access = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'keyholders');
		
		if ($access == 0) { // no access 
			return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
				'message'  => 'User '. $usr->getName().' has no access to keyholders information',
			));		
		}
		else {
			return $this->render('AcmeSosMonBundle:Keyholder:keyholders.html.twig', 
				array('access' => $access)
			);
		}
    }
	
/**
* list of keyholders as array	
* /data
* @return json array of categories
*/
    public function dataAction($opt)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'keyholders');

		if ($access == 0) { // guest cannot see any users
			return new Response(json_encode(array()));
		}
		else {	
			return new Response(json_encode(
				$em->getRepository('AcmeSosMonBundle:Keyholder')->getList( $usr->getOwnid(), $opt )
			));
		}
    }	
	
/**
* create or edit keyholder
* /entity
* @param int $id
* @return view
*/
	public function entityAction($id, Request $request)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'keyholders');

		if ($access == 0) { // no access 
			return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
				'message'  => 'User '. $usr->getName().' has no access to keyholders information',
			));		
		}
		
		$entity = $em->getRepository('AcmeSosMonBundle:Keyholder')->findEntity($id, $usr->getOwnid());
		$entity['access'] = $access;
		$entity['free_ibuttons'] = $em->getRepository('AcmeSosMonBundle:Ibutton')->getAvailableIbuttons($usr->getOwnid(), $id);
		
		$form = $this->createForm(new KeyholderType(), $entity);
				
		return $this->render('AcmeSosMonBundle:Keyholder:keyholder.html.twig', array(
			'form'   => $form->createView(), 
			'entity' => $entity,
		));
    }

/**
* save Keyholder
* /save
* @param int $id
* @return json array('id','message','success')
*/
	public function saveAction(Request $request)
	{
		if ($request->getMethod() == 'POST') {

			$usr= $this->get('security.context')->getToken()->getUser();
			$em = $this->getDoctrine()->getManager();
			$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'keyholders');

			if ($access == 0) { // no access 
				return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
					'message'  => 'User '. $usr->getName().' can not save keyholder information',
				));		
			}
		
			$var = $request->request->all();	
			$params = $var['keyholder'];
			$id = $em->getRepository('AcmeSosMonBundle:Keyholder')->saveEntity( $params );
			
			$this->get('call_actlog')->regActlog($usr->getOwnid(), $usr->getId(),
					'Key holder "'.$params['lname'].' '.$params['fname'].'"'.((int)$params['id'] == 0 ? ' created' :' modified'), 'KEYHOLDERS', $id, 0);		
													
			return new Response(Resanswer::getRetJSON('',$id > 0, $id));	
		}	
	}	
	
/**
* delete Keyholder (set deleted = 1)	
* /delete
* @param int $id
* @return json array('id','message','success')
	*/	
	public function deleteAction($id)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'keyholders');
		if ($access == 0) { // no access 
			return new Response(Resanswer::getRetJSON('User '. $usr->getName().' can not delete keyholder',false, $id));
		}	
		
		$entity = $em->getRepository('AcmeSosMonBundle:Keyholder')->deleteEntity($id );

		if ($entity) {
			$this->get('call_actlog')->regActlog($usr->getOwnid(), $usr->getId(),'Keyholder "'.$entity->getLname().' '.$entity->getFname().'" deleted', 'KEYHOLDERS', $id, 0);		
		}
						
		return new Response(Resanswer::getRetJSON('',true, $id));
    }	
	
/**
* translist
* /translist
* @return view
*/
    public function translistAction()
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'keyholders');

		if ($access == 0) { // guest cannot see any buttons
			return new Response(json_encode(array()));
		}
		else {
			return new Response(json_encode(
				$em->getRepository('AcmeSosMonBundle:Keyholder')->getTransferList( $usr->getOwnid() )
			));
		}
    }	

}
