<?php

namespace Acme\SosMonBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Acme\SosMonBundle\Form\Type\StationType;
use Acme\SosMonBundle\Model\Resanswer;
use Acme\SosMonBundle\Model\Common;

class StationsController extends Controller
{
	
/**
* stations main screen: list of available stations
* /index
* @return view
*/
    public function indexAction()
    {
		$usr= $this->get('security.context')->getToken()->getUser();	
		$access = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'stations');
		
		if ($access == 0) { // no access 
			return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
				'message'  => 'User '. $usr->getName().' has no access to stations information',
			));		
		}
		else {
			$company = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:Customer')->find( $usr->getOwnid() );
				
			return $this->render('AcmeSosMonBundle:Station:stations.html.twig', 
				array('access' => $access,
					'x_center' => ($company) ? $company->getDefLat() : 0,
					'y_center' => ($company) ? $company->getDefLong() : 0,)
			);
		}
    }
	
/**
* list of stations as array	
* /data
* @return json array 
*/
    public function dataAction($opt, Request $request)
    {
	
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'stations');

		if ($access == 0) { // guest cannot see any users
			return new Response(json_encode(array()));
		}
		else {
			$var = $request->request->all();

			return new Response(json_encode(
					$em->getRepository('AcmeSosMonBundle:Station')->getList( $usr->getId(), $opt, isset($var['alertsystemid'])? (int)$var['alertsystemid'] : 0 )
				));
		}
    }	
	
/**
* list of stations as tree array	
* /data
* @return json array 
*/
    public function treedataAction(Request $request)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'stations');

		if ($access == 0) { // guest cannot see any users
			return new Response(json_encode(array()));
		}
		else {
			$var = $request->request->all();
			$view = isset($var['view']) ? (int)$var['view'] : 0;
			if ($view ==  0) {
				$list = $em->getRepository('AcmeSosMonBundle:Station')->getTree( $usr->getId() );	
			}
			else {
				$list = $em->getRepository('AcmeSosMonBundle:Station')->getList( $usr->getId(), 0 );	
			}
			return new Response(json_encode($list));
		}
    }	
/**
* create or edit station
* /entity
* @param int $id
* @return view
*/
	public function entityAction($id, Request $request)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'stations');

		if ($access == 0) { // no access 
			return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
				'message'  => 'User '. $usr->getName().' has no access to stations information',
			));		
		}
		
		$entity = $em->getRepository('AcmeSosMonBundle:Station')->findEntity($id, 
											$em->getRepository('AcmeSosMonBundle:Customer')->find( $usr->getOwnid() ));
		$entity['access'] = $access;
		$entity['statuses']      = Common::getUnitStatuses();//   $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:Station')->getUnitStatuses();
		$entity['types']         = $em->getRepository('AcmeSosMonBundle:Station')->getUnitTypes();
		$entity['pricing_list']  = $em->getRepository('AcmeSosMonBundle:Station')->getUnitPricing();
		$entity['devent']        = $em->getRepository('AcmeSosMonBundle:Event')->eventDescription($entity['eventid']);
		$entity['locations']     = $em->getRepository('AcmeSosMonBundle:Alertsystem')->getListCombo($usr->getOwnid(), 2);
		$alertsys = $em->getRepository('AcmeSosMonBundle:Alertstations')->getLocation( $id );
		$entity['alertsystemid'] = $alertsys['id'];
		
		$form = $this->createForm(new StationType(), $entity);
				
		return $this->render('AcmeSosMonBundle:Station:station.html.twig', array(
			'form'   => $form->createView(), 
			'entity' => $entity,
		));
    }

/**
* id station
* /entity_id
* @param int $id
* @return view
*/
	public function idAction($id)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$entity = $em->getRepository('AcmeSosMonBundle:Station')->findEntity($id, $em->getRepository('AcmeSosMonBundle:Customer')->find( $usr->getOwnid() ));

		return new Response(json_encode($entity));		
    }	
	
/**
* info station
* /entity
* @param int $id
* @return view
*/
	public function infoAction($id, Request $request)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$entity = $em->getRepository('AcmeSosMonBundle:Station')->findEntity($id, $em->getRepository('AcmeSosMonBundle:Customer')->find( $usr->getOwnid() ));

		$entity['statuses'] = Common::getUnitStatuses();
		$entity['types']    = $em->getRepository('AcmeSosMonBundle:Station')->getUnitTypes();
		$entity['devent']   = $em->getRepository('AcmeSosMonBundle:Event')->eventDescription( $entity['eventid'] );		
		return $this->render('AcmeSosMonBundle:Station:station_ro.html.twig', array(
			'entity' => $entity,
		));
    }

/**
* save Station
* /save
* @param int $id
* @return json array('id','message','success')
*/
	public function saveAction(Request $request)
	{
		if ($request->getMethod() == 'POST') {

			$usr= $this->get('security.context')->getToken()->getUser();
			$em = $this->getDoctrine()->getManager();
			$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'stations');

			if ($access == 0) { // no access 
				return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
					'message'  => 'User '. $usr->getName().' can not save station information',
				));		
			}
		
			$var = $request->request->all();		
			$params = $var['station'];
			
			$check = $em->getRepository('AcmeSosMonBundle:Station')->checkEntity( $params );
			if ($check['success'] == true) {			
			
				$id = $em->getRepository('AcmeSosMonBundle:Station')->saveEntity( $params );			
				$em->getRepository('AcmeSosMonBundle:Alertstations')->setLocation( $id, $params['asystems'] );
				$em->getRepository('AcmeSosMonBundle:Keystations')->assignKeys( $id, explode( ',', $params['keys'] ) );
			
				$this->get('call_actlog')->regActlog($usr->getOwnid(), $usr->getId(),
						'Station "'.$params['name'].'"'.((int)$params['id'] == 0 ? ' created' :' modified'), 'STATIONS', $id, 0);
				
				return new Response(Resanswer::getRetJSON('',$id > 0, $id));						
			}
			else {
				return new Response(Resanswer::getRetJSON($check['message'],false, 0));			
			}
		}	
	}	
/**
* active Stations 	
* /delete
* @param int $id
* @return json array('id','message','success')
	
	public function activeAction($id)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$access = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'stations');
		if ($access == 0) { // no access 
			return new Response(Resanswer::getRetJSON('User '. $usr->getName().' can not delete station',false, $id));
		}	
		
		$this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:Station')->activeEntity( $id );			
		return new Response(Resanswer::getRetJSON('',true, $id));
    }		
	*/	
/**
* delete Stations 	
* /delete
* @param int $id
* @return json array('id','message','success')
	*/	
	public function deleteAction($id)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'stations');
		if ($access == 0) { // no access 
			return new Response(Resanswer::getRetJSON('User '. $usr->getName().' can not delete station',false, $id));
		}	
		
		$entity = $em->getRepository('AcmeSosMonBundle:Station')->deleteEntity($id );	
		
		if ($entity) {
			$this->get('call_actlog')->regActlog($usr->getOwnid(), $usr->getId(),'Station "'.$entity['name'].'" deleted', 'STATIONS', $id, 0);		
		}		
						
		return new Response(Resanswer::getRetJSON('',true, $id));
    }	

}
