<?php

namespace Acme\SosMonBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Acme\SosMonBundle\Form\Type\UserType;
use Acme\SosMonBundle\Model\Resanswer;
use Acme\SosMonBundle\Model\Common;

class UsersController extends Controller
{
	
/**
* users main screen: list of available users
* /index
* @return view
*/
    public function indexAction()
    {
		$usr= $this->get('security.context')->getToken()->getUser();	
		$access = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'users');
		
		if ($access == 0) { // no access 
			return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
				'message'  => 'User '. $usr->getName().' has no access to Web users information',
			));		
		}
		else {
			return $this->render('AcmeSosMonBundle:User:users.html.twig', 
				array('access' => $access,
					  'ownid'  => $usr->getOwnid())
			);
		}
    }
	
/**
* list of users as array	
* /data
* @return json array of user
*/
    public function dataAction($opt)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$rep = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User');
		$access = $rep->getSectionAccess( $usr->getAccword(), 'users');

		if ($access == 0) { // guest cannot see any users
			return new Response(json_encode(array()));
		}
		else {	
			return new Response(
				json_encode($rep->getList( $usr->getOwnid(), $opt ))
			);
		}
    }	
	
/**
* list of users as array for company
* /data
* @return json array of categories
*/
    public function users_custAction(Request $request)
    {
		$var = $request->request->all();		
		$ownid = isset( $var['ownid'] ) ? (int)$var['ownid'] : 0;	
		
		return new Response(json_encode(
				$this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getUsersForCompany( $ownid )
				));
    }	
	
/**
* create or edit user
* /entity
* @param int $id
* @return view
*/
	public function entityAction($id, Request $request)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$em = $this->getDoctrine()->getManager();
		$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'users');

		if ($access == 0) { // no access 
			return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
				'message'  => 'User '. $usr->getName().' has no access to Web users information',
			));		
		}
		
		$company = $em->getRepository('AcmeSosMonBundle:Customer')->find( $usr->getOwnid() );

		$entity = $em->getRepository('AcmeSosMonBundle:User')->findEntity($id);
		$entity['access']   = Common::getAccessTypes(9);
		$entity['ownid']    = $usr->getOwnid();
		$entity['alertsystems'] = '';
		if ($usr->getOwnid() == 0) // global admin
		{
			$entity['prefix']   = '';	
			$entity['contacts'] = '';
		}
		else {
			$entity['prefix'] = $company->getPrefix().'_';
			$entity['login']  = substr($entity['login'], strlen($entity['prefix']), strlen($entity['login']));
			$entity['contacts'] = $em->getRepository('AcmeSosMonBundle:Contact')->getListForCombo($usr->getOwnid(), 2);			
		}
		
		$form = $this->createForm(new UserType(), $entity);
				
		return $this->render('AcmeSosMonBundle:User:user.html.twig', array(
			'form'   => $form->createView(), 
			'entity' => $entity,
			'prefix' => $entity['prefix'],			
			'access' => $access,

		));
    }

/**
* systems for user
* /systems
* @return array
*/
	public function systemsAction(Request $request)
    {	
		$var = $request->request->all();
		$id    = isset( $var['id'] )    ? (int)$var['id'] : 0;
		$ownid = isset( $var['ownid'] ) ? (int)$var['ownid'] : 0;

		return new Response(json_encode( 
				$this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getSystems( $id, $ownid ) 
			));
    }
	
/**
* systems for user
* /systems
* @return array
*/
	public function newadminAction($id, Request $request)
    {	
		$em = $this->getDoctrine()->getManager();
		$company = $em->getRepository('AcmeSosMonBundle:Customer')->findEntity($id);
		
		$admin = $em->getRepository('AcmeSosMonBundle:User')->createAdmin( $id, $company['prefix'], $this->container->get('security.password_encoder') );	
		
		$message = 'Temporary administrator user was created. Please copy credentials. Login:"'.$admin['prefix'].$admin['login'].'";Password:"'.$admin['password'].'"';
		return new Response(Resanswer::getRetJSON($message, $admin['userid'] > 0, $admin['userid']));		
	}
	
/**
* access for user
* /access
* @return array
*/
	public function accessAction(Request $request)
    {	
		$var = $request->request->all();
		$accword = isset( $var['accstr'] ) ? (string)$var['accstr'] : str_repeat("0", 10);
		
		return new Response(json_encode( 
				$this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->getAccess( $accword ) 
			));		
    }
	
	
/**
* save User
* /save
* @param int $id
* @return json array('id','message','success')
*/
	public function saveAction(Request $request)
	{
		if ($request->getMethod() == 'POST') {

			$usr= $this->get('security.context')->getToken()->getUser();
			$em = $this->getDoctrine()->getManager();
			$access = $em->getRepository('AcmeSosMonBundle:User')->getSectionAccess( $usr->getAccword(), 'users');

			if ($access == 0) { // no access 
				return $this->render('AcmeSosMonBundle:Default:denied.html.twig', array(
					'message'  => 'User '. $usr->getName().' can not save Web users information',
				));		
			}
					
			$var = $request->request->all();				 
			$params = $var['user'];
			
			if (isset($params['acclevel']) == false) {
				$params['acclevel'] = 1;	
			}
			
			if (isset($params['alertcontactid']) == false) {
				$params['alertcontactid'] = 0;	
			}	
			
			$id = (int)$params['userid'];
			$message = '';
			if ( $em->getRepository('AcmeSosMonBundle:User')->checkEntity( $id, $params['prefix'].$params['login'] ) ) {
			
				$id = $em->getRepository('AcmeSosMonBundle:User')->saveEntity( $params, $this->container->get('security.password_encoder'));
			
				$em->getRepository('AcmeSosMonBundle:Alertusers')->assignSystems( $id, explode( ',', $params['asystems'] ) );
			
				$this->get('call_actlog')->regActlog( $usr->getOwnid(), $usr->getId(),
												 'User "'.$params['name'].'"'.((int)$params['userid'] == 0 ? ' created' :' modified'),
												 'USERS', $id, 0);				
			}
			else {
				$message = 'Login "'.$params['prefix'].$params['login'].'" is already exists';
			}
			return new Response(Resanswer::getRetJSON($message, $id > 0, $id));	
		}	
	}

/**
* change user password
* /change_password
* @param int $id
* @return view
*/
	public function change_passwordAction($id, Request $request)
    {
		$entity = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->findEntity($id);	
		$form = $this->createForm(new ChgpwdType(), $entity);
				
		return $this->render('AcmeSosMonBundle:User:change.html.twig', array(
			'form'   => $form->createView(), 
			'userid' => $entity['userid'],
		));
    }	
	
	
/**
* save changed password. Is not using by now
* /save_password
* @param int $id
* @return json array('id','message','success')
*/	
	public function save_passwordAction(Request $request)
	{
		if ($request->getMethod() == 'POST') {
		
			$var = $request->request->all();
			
			if ( isset($var['chgpwd']['newpassword']) ){
				$usr= $this->get('security.context')->getToken()->getUser();
				$this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->savePassword( $usr->getId(), $var['chgpwd']['newpassword']);
			}
			return $this->redirect($this->generateUrl('acme_sosmon_homepage'));		
		}	
	}	


	public function changeAction($id, Request $request)
    {		
		$this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User')->chgPasswordEntity($id, 1);					
		return new Response(Resanswer::getRetJSON('',true, $id));	
    }
	
/**
* active User
* /active
* @param int $id
* @return json array('id','message','success')
*/	
	public function activeAction($id)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$rep = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User');
		$access = $rep->getSectionAccess( $usr->getAccword(), 'users');
		
		if ($access == 0) { // no access 
			return new Response(Resanswer::getRetJSON('User '. $usr->getName().' can not activate/deactivate Web users',false, $id));
		}		
		
		$rep->activeEntity($id);		
		return new Response(Resanswer::getRetJSON('',true, $id));
    }	
	
/**
* delete User (set deleted = 1)	
* /delete
* @param int $id
* @return json array('id','message','success')
	*/	
	public function deleteAction($id)
    {
		$usr= $this->get('security.context')->getToken()->getUser();
		$rep = $this->getDoctrine()->getManager()->getRepository('AcmeSosMonBundle:User');
		
		$access = $rep->getSectionAccess( $usr->getAccword(), 'users');
		if ($access == 0) { // no access 
			return new Response(Resanswer::getRetJSON('User '. $usr->getName().' can not delete Web users',false, $id));
		}	
		$entity = $rep->deleteEntity($id );	
		
		if ($entity) {
			$this->get('call_actlog')->regActlog($usr->getOwnid(), $usr->getId(),'User "'.$entity->getName().'" deleted', 'USERS', $id, 0);		
		}
					
		return new Response(Resanswer::getRetJSON('',true, $id));
    }	

}
