<?php
namespace Acme\SosMonBundle\Entity;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\NoResultException;
use Acme\SosMonBundle\Model\Common;
use Acme\SosMonBundle\Model\CampaignStatus;
use \DateTime;

class AlertcampaignRepository extends EntityRepository 
{
	/*
	*	Create campaign
	*	@param int $eventid
	*	@param int $stationid 
	*	
	*	@return int count
	*/
	public function createCampaign( $eventid, $stationid, $message = '' )
	{	
		$alert = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Alertstations')->getLocation( $stationid );
		$contacts = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Alertcontacts')->getContacts($alert['id'], 0);	
		
		$count = count( $contacts );
		foreach( $contacts as $contact) {
			$this->saveEntity( array('eventid'   => $eventid,
									 'contactid' => $contact['id'],
									 'nr' 		 => $contact['nr'],
									 'phone' 	 => str_replace(" ", "", $contact['phone1']),
									 'sname'     => $alert['name'].' '.$message,
									 'calls'     => 0,
									 'deleted'   => 0,
									 'status' 	 => 0)
			);										
		}
		return $count;
	}
	
	/*
	*	Get first campaign
	*
	*	@param int $eventid
	*	@return CampaignStatus
	*/
	public function getFirstForCampaign( $eventid )
	{
		$result =  new CampaignStatus();		
						
		$rows = $this->findBy(  array('eventid'=> $eventid, 'deleted' => 0), 
							    array('nr' => 'ASC')
				);	

		foreach($rows as $row) {
			if ( $this->goodForCall( $row->getPhone() ) ) {
				
				$this->statusUpdate( $row->getId() , $this->call_status_current() );  // next to call
				
				$result->setPhone( $row->getPhone() )
					   ->setContactid( $row->getContactid() )
					   ->setEventid( $row->getEventid() )
					   ->setCalls( $row->getCalls() )
					   ->setMessage( $row->getSname() )
					   ->setFound(true);
				break;
			}
		}				
		return $result;					
	}
	
	/*
	*	Get next campaign
	*   @param string $phone
	*
	*	@return CampaignStatus
	*/
	public function getNextForCampaign( $phone, $logger = null )
	{
		$result =  new CampaignStatus();
		
		$curr = $this->findOneBy(  array('phone'=> $phone,'status' => $this->call_status_current(), 'deleted' => 0));	
		
		if ($curr) {
			$this->statusUpdate( $curr->getId(), $this->call_status_failed() );  // call and fail
			
			$rows = $this->findBy(  array('eventid'=> $curr->getEventid(), 'deleted' => 0 ), 
								    array('nr' => 'ASC')
					);	
					
			$maxcall_excepton = false;
			
			$info = '1';
			
			foreach($rows as $row) {
				
				if ( (( $row->getNr() > $curr->getNr() ) || (count($rows) == 1)) 
						&& ($this->goodForCall( $row->getPhone() ))) {
					// first greater nr
					$info .= '2';				
					if ( $row->getCalls() < $this->call_max() ) { 
						$info .= '3('.$row->getCalls().':'.$this->call_max().')';
						$this->statusUpdate( $curr->getId(), $this->call_status_failed() );  // call and fail
						$this->statusUpdate( $row->getId() , $this->call_status_current() );  // next to call
					
						$result->setPhone( $row->getPhone() )
								->setContactid( $row->getContactid() )
								->setEventid( $row->getEventid() )
								->setMessage(  $row->getSname() )
								->setFound(true)
								->setCalls( $row->getCalls() )
								->setVar('Keep'.$info);
					}
					else {
						$maxcall_excepton = true; // have to be stoped - too much calls
					}
					break;
				}
			}
			
		//	$result->setVar( 'VAR'.json_encode($rows) );
			
			if ( $maxcall_excepton ) { // stop !
				$this->statusUpdate( $curr->getId(), $this->call_status_failed() );  // call and fail
				return $this->stopCampaign( $phone );
			}
			
			if ( !$result->getFound() ) { // roll over begin
				$this->statusUpdate( $curr->getId(), $this->call_status_failed() ); // call and fail
				$result = $this->getFirstForCampaign( $curr->getEventid() );
				$result->setVar('New campaign'.$info);
			}
		}
				
		return $result;
	}	

	/*
	*	Stop campaign
	*	@param string $phone
	*	@return CampaignStatus
	*/
	public function stopCampaign( $phone )
	{
		$result =  new CampaignStatus();
						
		$curr = $this->findOneBy(  array('phone'=> $phone, 'status' => $this->call_status_current(), 'deleted' => 0));	
		if ($curr) {					
			$result->setPhone( $curr->getPhone() )
					->setContactid( $curr->getContactid() )
					->setEventid( $curr->getEventid() )
					->setFound(true);
			
			$this->removeCampaign($curr->getEventid());				
		}
		return $result;
	}	
	/*
	*	Stop campaign
	*	@param int $eventid
	*	@return Bool
	*/	
	public function removeCampaign( $eventid )
	{
		$em = $this->getEntityManager();
		$rows = $this->findBy(  array('eventid'=> $eventid, 'deleted' => 0)	);	

		foreach($rows as $row) {
			$row->setDeleted( 1 );
			$em->persist( $row );
		}
		$em->flush();
		return true;
	}	
	
	/*
	*	goodForCall : check if phone is not busy
	*
	*	@param string $phone
	*	@return boolean
	*/
	public function goodForCall( $phone )
	{
		$curr = $this->findOneBy(  array('phone'=> $phone, 'status' => $this->call_status_current(), 'deleted' => 0));	
		return ($curr) ? false : true;
	}	
	
	/**
	* Create new Alertcampaign
	*
	* @param array $var  
	*
	* @return entity id
	*/	
	public function saveEntity( $var )
	{
		$entity = new Alertcampaign();
		
		$entity->setEventid(   $var['eventid'] );
		$entity->setContactid( $var['contactid'] );
		$entity->setNr(        $var['nr'] );
		$entity->setStatus(    $var['status'] );		
		$entity->setPhone(     $var['phone'] );	
		$entity->setDeleted(   $var['deleted'] );	
		$entity->setCalls(     $var['calls'] );	
		$entity->setSname(     $var['sname'] );		
		$entity->setModtime( new DateTime);
					
		$em = $this->getEntityManager();				
		$em->persist($entity);
		$em->flush();
		return $entity->getId();			
	}	
	
	/**
	* statusUpdate. Count current calls during campaign
	*
	* @param integer $id
	* @param integer $status (0 - not called; 1 - in call; 2 - called but failed
	* @return entity id
	*/	
	public function statusUpdate( $id, $status )
	{
		$entity = $this->find( $id );
		if ($entity) {
			$entity->setStatus( $status );
			if ( $status == $this->call_status_current() ) {
				$entity->setCalls( $entity->getCalls() + 1 );
			}
			$entity->setModtime( new DateTime );
		
			$em = $this->getEntityManager();				
			$em->persist($entity);
			$em->flush();
		}
		return $id;			
	}
	/**
	* getHanging. Check hanging campaign
	*
	* @return array
	*/		
	public function getHanging()
	{
		$date = new DateTime();
		$date->modify("-10 minutes");
		
		$query = $this->getEntityManager()->createQuery(
			'SELECT DISTINCT p.eventid
			FROM AcmeSosMonBundle:Alertcampaign p
			WHERE p.deleted = 0 AND p.status = 1 AND p.modtime < :time
			GROUP BY p.eventid
			ORDER BY p.eventid'
		)->setParameter('time', $date);	
		
		$campaigns = $query->getResult();
		
		return $campaigns;
	}

	public function call_status_current()
	{
		return 1;
	}
	
	public function call_status_failed()
	{
		return 2;
	}
	
	public function call_max()
	{
		return 3;
	}	

	

}