<?php
namespace Acme\SosMonBundle\Entity;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\NoResultException;
use Acme\SosMonBundle\Model\Common;

class AlertcontactsRepository extends EntityRepository 
{
/*
	List of selected contacts for system
	@params int $id
	@return array
*/
	public function getContacts($id, $ownid)
	{

		$contacts = $this->findByAlertsystemid( $id, array('nr' => 'ASC' ));  
		$res = array();	
		
		foreach ($contacts as $row) {
			$contact = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Contact')->findEntity($row->getContactid(), $ownid);
			$contact['nr'] = $row->getNr();
			$res[] = $contact;
		}
		return $res;
	}

/*
	List of alert system and selected are checked
*/

	public function getAlerts($ownid, $id)
	{
		$systems = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Alertsystem')->getList( $ownid );
		
		foreach ($systems as $row) {
			$asys = $this->findBy( array( 'alertsystemid' => $row['id'],
										  'contactid'     => $id));
										  
			$res[] = array('id'    => $row['id'],
							'name' => $row['name'],
							'chk'  => (($asys)?true: false), 
							);
		}
		return $res;
	}
	
	/**
	* getDescriptionForContact - description alertsystems for user
	*
	* @return string
	*/
	public function getDescriptionForContact( $contactid )	
	{
		$stmt = $this->getEntityManager()->getConnection()->prepare('SELECT a.name FROM alertcontacts c, alertsystems a WHERE c.alertsystemid = a.id and c.contactid = :contactid ORDER BY a.name' );
		$stmt->bindValue('contactid', $contactid);
		$stmt->execute();
		$rows = $stmt->fetchAll();
		if (count( $rows ) == 0 ) {
			$descr = 'none';
		}
		else {
			$row = current($rows);
			$descr = $row['name'];
		}
		
		if (count( $rows ) > 1) {
			$descr .= ' and '.(count( $rows )-1).' more';
		}
		
		return $descr;
	}
	
/*
	Renumerate contact order for system 
	@params int $id
	@return 1
*/
	public function renumerateSystem( $id )
	{
		$em = $this->getEntityManager();				
		
		$contacts = $this->findByAlertsystemid( $id, array('nr' => 'ASC' ));  
		$nr = 1;
		foreach ($contacts as $entity) {
			$entity->setNr( $nr );
			$em->persist($entity);
			$nr++;			
		}
		$em->flush();

		return 1;
	}
	
	
	/**
	* Assign systems to contact
	*
	* @return 1
	*/
	public function assignSystems( $id, $asystems )
	{	
		$this->deleteForContact( $id );	
		foreach($asystems as $row) {
			$this->saveEntity( array (  'contactid'     => $id,
										'alertsystemid' => $row,
										'nr'            => 9999) ); // goes last
			$this->renumerateSystem( $row );										
		}	
		return 1;
	}
	
	/**
	* Assign contacts for system
	*
	* @return $res array
	*/
	public function assignContacts( $id, $contacts )
	{	
		$this->deleteForSystem( $id );
		$nr = 1;
		foreach($contacts as $row) {
			if ((int)$row > 0) {
				$this->saveEntity( array (  'contactid'     => $row,
										'alertsystemid' => $id,
										'nr'            => $nr) );
				$nr++;										
			}
		}	
		return 1;
	}	

	/**
	* Save Alertcontact
	*
	* @param array $var  

	* @return entity id
	*/	
	public function saveEntity( $var )
	{
		$id = 0;
		if ((int)$var['contactid'] > 0) {
			$entity = new Alertcontacts();
		
			$entity->setContactid(     $var['contactid'] );
			$entity->setAlertsystemid( $var['alertsystemid'] );
			$entity->setNr( $var['nr'] );
					
			$em = $this->getEntityManager();				
			$em->persist($entity);
			$em->flush();
			$id = $entity->getId();	
		}		
		return $id;
	}

	/**
	* Delete for contact
	*
	* @param int $id
	* @return 1
	**/
	public function deleteForContact( $id )
	{		
		$this->getEntityManager()->getConnection()->executeUpdate('delete from alertcontacts where contactid = :id', 
			array('id' => $id)
		);			 
		return 1;
	}
	
	/**
	* Delete for contact
	*
	* @param int $id
	* @return 1
	**/
	public function deleteForSystem( $id )
	{		
		$this->getEntityManager()->getConnection()->executeUpdate('delete from alertcontacts where alertsystemid = :id', 
			array('id' => $id)
		);			 
		return 1;
	}	
}