<?php
namespace Acme\SosMonBundle\Entity;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\NoResultException;

class ContactRepository extends EntityRepository 
{

	/**
	* Get all Contacts
	*
	* @param int $ownid
	* @param int $opt
	* @return $res array
	*/
	public function getList($ownid, $opt)
	{	
		$contacts = $this->findBy(array('ownid' => $ownid, 'deleted' => 0), array('priority' => 'ASC','lname' => 'ASC'));

		$res = array();
		if ($opt == 2) {
			$res[] = array(
				'id' 		=> 0,			
				'name' 		=> '< NONE >',	
			);		
		}
		foreach($contacts as $cont) {  
			$res[] = array(
				'id' 		=> $cont->getId(),
				'name'      => $cont->getLname().' '.$cont->getFname(),			
				'fname' 	=> $cont->getFname(),
				'lname' 	=> $cont->getLname(),				
				'phone1' 	=> $cont->getPhone1(),
				'phone2' 	=> $cont->getPhone2(),
				'phone3' 	=> $cont->getPhone3(),
				'phone4' 	=> $cont->getPhone4(),
				'priority'	=> $cont->getPriority(),
				'sms' 		=> $cont->getSms(),
				'email' 	=> $cont->getEmail(),
				'alertsystems' => $this->getEntityManager()->getRepository('AcmeSosMonBundle:Alertcontacts')->getDescriptionForContact( $cont->getId() ),
			);		
		}

		return $res;
	}	
	
	/**
	* Get all Contacts
	*
	* @param int $ownid
	* @param int $opt
	* @return $res array
	*/
	public function getListForCombo($ownid, $opt)
	{	
		$contacts = $this->findBy(array('ownid' => $ownid, 'deleted' => 0), array('priority' => 'ASC','lname' => 'ASC'));

		$res = array();
		if ($opt == 2) {
			$res[ 0 ] = '< NONE >';	
		}
		foreach($contacts as $cont) {  
			$res[ $cont->getId() ] = $cont->getLname().' '.$cont->getFname();	
		}

		return $res;
	}	

	/**
	* Get entity SosMon contact
	*
	* @param int $id
	* @param int $ownid	
	* @return entity $entity
	*/			
	public function findEntity($id, $ownid)
	{
		$entity = $this->find($id);	
		if (!$entity) {
			$entity = new Contact(); 
			$entity->setOwnid( $ownid );
		}
		
		return  array(	'id'		=> (int)$entity->getId(),			
						'ownid' 	=> (int)$entity->getOwnid(),			
						'priority' 	=> (int)$entity->getPriority(),		
						'lname' 	=> $entity->getLname(),
						'fname' 	=> $entity->getFname(),
						'sms' 		=> $entity->getSms(),
						'email' 	=> $entity->getEmail(),		
						'phone1' 	=> $entity->getPhone1(),	
						'phone2' 	=> $entity->getPhone2(),
						'phone3' 	=> $entity->getPhone3(),
						'phone4' 	=> $entity->getPhone4(),
					);		
	}
	
	/**
	* Check phone is unque Contact
	*
	* @param array $var  
	* @return boolean
	*/
	public function checkPhone( $var )
	{
		$id    = isset($var['id'])    ? (int)$var['id']    : 0;
		$ownid = isset($var['ownid']) ? (int)$var['ownid'] : 0;
		
		$items = $this->findBy( array( 'phone1' => $var['phone1'], 'ownid' => $ownid, 'deleted' => 0 ));	
		
		$isgood = true;		
		foreach($items as $item) {
			if ($item->getId() != $id ) {
				$isgood = false;	
			}
		}
		
		return ($isgood);		
	}
	
	/**
	* Save Contact
	*
	* @param array $var  
	* @return entity id
	*/
	public function saveEntity( $var )
	{
		$entity = $this->find( (int)$var['id'] );	
		if (!$entity) {
			$entity = new Contact();
		}
	
		try {
			$entity->setPriority((int)$var['priority']);		
			$entity->setOwnid(   (int)$var['ownid']);
			$entity->setFname(   $var['fname']);			
			$entity->setLname(   $var['lname']);	
			$entity->setPhone1(   $var['phone1']);		
			$entity->setPhone2(   $var['phone2']);		
			$entity->setPhone3(   $var['phone3']);		
			$entity->setPhone4(   $var['phone4']);
			$entity->setEmail(   $var['email']);
			$entity->setSms(     $var['sms']);
					
			$em = $this->getEntityManager();				
			$em->persist($entity);
			$em->flush();
			$id = $entity->getId();
		}
		catch (Exception $e) {
			$id = -1;
		}
		return $id;			
	}	
	
	/**
	* Delete contact
	*
	* @param int $id
	* @return 1
	**/
	public function deleteEntity( $id )
	{	
	
		$entity = $this->find( $id );	
		if ($entity) {
			$em = $this->getEntityManager();						
			$entity->setDeleted( 1 ); 
			$em->persist($entity);				
			$em->flush();		
	    }
		return $entity;
	}	
	

	
}