<?php
namespace Acme\SosMonBundle\Entity;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\NoResultException;

class CustomerRepository extends EntityRepository 
{

	/**
	* Get all Customers
	* @param string $opt
	* @return $res array
	*/
	public function getList($opt = '')
	{	
		$customers = $this->findByDeleted(0, array('active' => 'ASC','name' => 'ASC'));
		
		$res = array();
		if ($opt == 'A') {
			$res[] = array(
				'ownid' 	=> 0,			
				'name' 		=> 'ALL',	
				'prefix' 	=> '',
				'active' 	=> 0,
			);	
		}
		
		if ($opt == 'N') {
			$res[] = array(
				'ownid' 	=> 0,			
				'name' 		=> 'NONE',	
				'prefix' 	=> '',
				'active' 	=> 0,
			);	
		}		
		
		foreach ($customers as $cust) {
			$res[] = array(
				'ownid' 	=> $cust->getOwnid(),			
				'name' 		=> $cust->getName(),	
				'prefix' 	=> $cust->getPrefix(),
				'active' 	=> $cust->getActive(),
				);
			}
		return $res;
	}	

	/**
	* Get entity SosMon customer
	*
	* @param int $id
	* @return entity $entity
	*/			
	public function findEntity($id)
	{
		$entity = $this->find($id);	
		if (!$entity) {
			$entity = new Customer(); 
		}
		
		return array(	'ownid' 	=> (int)$entity->getOwnid(),			
						'name' 		=> $entity->getName(),
						'shipping'  => $entity->getShipping(),
						'billing'   => $entity->getBilling(),
						'contact_name'    => $entity->getContact_name(),
						'contact_pos'     => $entity->getContact_pos(),
						'contact_phone'   => $entity->getContact_phone(),
						'contact_mobile'  => $entity->getContact_mobile(),
						'contact_email'   => $entity->getContact_email(),
						'prefix' 	=> $entity->getPrefix(), 
						'keys' 		=> $entity->getKeys(),
						'sentry' 	=> $entity->getSentry(),		
						'rrate' 	=> $entity->getRrate(),						
						'deflat' 	=> $entity->getDeflat(),
						'deflong' 	=> $entity->getDeflong(),
						'active' 	=> $entity->getActive(),
						'mstype' 	=> $entity->getMstype(),
						'billcontact'=>$entity->getBillcontact(),						
						'fixedcount' 	 => $entity->getFixedcount(),
						'fixedprice' 	 => $entity->getFixedprice(),
						'extraprice' 	 => $entity->getExtraprice(),
					);		
	}

	/**
	* Save Customer
	*
	* @param array $var  
	* @return entity id
	*/
	public function saveEntity( $var )
	{	
		$entity = $this->find((int)$var['ownid']);	
		if (!$entity) {
			$entity = new Customer();
			$entity->setPrefix( $var['prefix'] );
		}
			
		try 
		{		
			$entity->setName(  (string) $var['name'] ); 
			$entity->setShipping(   (string) $var['shipping']);
			$entity->setBilling(    (string) $var['billing']);
			$entity->setContact_name(   (string) $var['contact_name']);
			$entity->setContact_pos(    (string) $var['contact_pos']);
			$entity->setContact_phone(  (string) $var['contact_phone']);
			$entity->setContact_mobile( (string) $var['contact_mobile']);
			$entity->setContact_email(  (string) $var['contact_email']);		
			$entity->setKeys(     (int)$var['keys']);		
			$entity->setSentry(   (int)$var['sentry']);			
			$entity->setRrate(    (int)$var['rrate']);			
			$entity->setDeflat(   (float)$var['deflat']);
			$entity->setDeflong(  (float)$var['deflong']);
			$entity->setActive(   (int)$var['active']);
			$entity->setMstype(   (int)$var['mstype']);
			$entity->setDeleted(  0 );
			$entity->setBillcontact(  $var['billcontact']);
			$entity->setFixedcount(   (int)$var['fixedcount']);
			$entity->setFixedprice(   (float)$var['fixedprice']);
			$entity->setExtraprice(   (float)$var['extraprice']);		
		}	
		catch (Exception $e) {
			return -1;
		}
				
		$em = $this->getEntityManager();				
		$em->persist($entity);
		$em->flush();
		return $entity->getOwnid();			
	}

	/**
	* Check customers prefix
	*
	* @param string $prefix
	* @return bool
	*/
	public function checkPrefix( $prefix )
	{	
		$customers = $this->findOneBy( array('prefix'  => $prefix,
										     'deleted' => 0));
		return ($customers)? false :true ;								  
	}
	
		
	/**
	* Customers set active status
	*
	* @param int $id
	* @param int $active
	* @return $id
	*/
	public function activeEntity( $id)
	{	
		$entity = $this->find( $id );	
		if ($entity) {
		    $active = $entity->getActive() == 0 ? 1 : 0;
			$entity->setActive( $active ); 
			$em = $this->getEntityManager();				
			$em->persist($entity);
			$em->flush();		
		}		 
		return $id;
	}	
	
	/**
	* Delete customer
	*
	* @param int $id
	* @return 1
	**/
	public function deleteEntity( $id )
	{	
		$entity = $this->find( $id );	
		if ($entity) {
			$em = $this->getEntityManager();						
			$entity->setDeleted( 1 ); 
			$em->persist($entity);				
			$em->flush();		
	    }		 								 
		return $entity;
	}	
		
	/**
	* Checks for default "test" customer 
	*
	**/
	public function getTestOwnid()
	{
		$prefix = 'test';
		if ($this->checkPrefix(0, $prefix)) {
			$this->saveEntity(  array( 'ownid'   => 0,
				'name'    => 'Test company',
				'shipping'  => '',
				'billing'   => '',
				'contact_name'    => '',
				'contact_pos'     => '',
				'contact_phone'   => '',
				'contact_mobile'  => '',
				'contact_email'   => '',							
				'keys' 	  => 0,
				'prefix'  => 'test',
				'sentry'  => 0,
				'deflat'  => 40,
				'deflong' => 50,
				'rrate'   => 0,
				'active'  => 0,
				'mstype'  => 0,
				'billcontact'=> '',						
				'fixedcount' 	=> 0,
				'fixedprice' 	=> 0,
				'extraprice' 	=> 0,				
			));		
		}
		
	
		$stmt = $this->getEntityManager()->getConnection()->prepare('SELECT ownid FROM customers where prefix = :prefix LIMIT 1');
		$stmt->bindValue('prefix', $prefix);
		$stmt->execute();
		$row = current($stmt->fetchAll());
		
		return $row['ownid'];
	}
	
	
}