<?php
namespace Acme\SosMonBundle\Entity;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\NoResultException;
use \DateTime;
use \PHPExcel;
use Acme\SosMonBundle\Model\Common;
use Acme\SosMonBundle\Model\Payload;
		
class EventRepository extends EntityRepository 
{

	/**
	* Get all Events
	* @param int $rows
	* @param int $offset
	* @param int $userid
	* @param int $systemid	
	* @param int $stationid
	* @param int $ibuttonid
	* @param int $keyholderid
	* @param int $status
	* @param String $dt1
	* @param String $dt2	
	* @return $res array
	*/
	public function getList($rows, $offset, $userid, $systemid,  $stationid, $ibuttonid, $keyholderid, $status, $dt1, $dt2)
	{	
		$rows = ($rows == 0)? 9999999 : $rows;
		$sql = 'SELECT * FROM getEvents(:userid, :stationid, :ibuttonid, :keyholderid, :status, :dt1, :dt2, :rows, :offset, :systemid)';
		$stmt = $this->getEntityManager()->getConnection()->prepare($sql);
		$stmt->bindValue('userid', $userid);		
		$stmt->bindValue('stationid', $stationid);
		$stmt->bindValue('ibuttonid', $ibuttonid);
		$stmt->bindValue('keyholderid', $keyholderid);
		$stmt->bindValue('status', $status);		
		$stmt->bindValue('dt1', $dt1);
		$stmt->bindValue('dt2', $dt2);
		$stmt->bindValue('rows', $rows);
		$stmt->bindValue('offset', $offset);
		$stmt->bindValue('systemid',$systemid);
		
		$this->getEntityManager()->getRepository('AcmeSosMonBundle:Sqllog')
			->saveEntity(array( 'userid' => $userid,
								'query'  => $sql,
								'params' => json_encode(array($userid,	$stationid,	$ibuttonid,	$keyholderid,$status, $dt1, $dt2, $rows,$offset)),
							  ));
																							  
		$stmt->execute();
		$rows = $stmt->fetchAll();			
		$total = 0;
		
		$res = array();
		$stats = Common::getUnitStatuses();	
		
		foreach ($rows as $row) {
			$total = (int)$row['total'];
			$res[] = array(
				'id' 			=> (int)$row['id'],		
				'ownid' 		=> (int)$row['ownid'],	
				'stationid' 	=> (int)$row['stationid'],				
				'sname' 		=> $row['sname'],	
				'esn' 			=> $row['esn'],
				'ibuttonid' 	=> (int)$row['ibuttonid'],				
				'iname' 		=> $row['iname'],	
				'keyholderid'	=> (int)$row['keyholderid'],				
				'kname' 		=> $row['kname'],		
				'longitude'		=> (float)$row['longitude'],				
				'latitude' 		=> (float)$row['latitude'],					
				'dt' 			=> $row['dt'],				
				'charge'		=> (int)$row['charge'],
				'status' 		=> $this->DisplayStatus( $row['status'], Payload::Decode9byte( $row['edata'] )),
				'newstatus'		=> $this->DisplayStatus2( Payload::Decode9byte( $row['edata'] )),
				'statname' 		=> $stats[ $row['status'] ],
				'edata' 		=> $row['edata'],	
				'location' 		=> $row['location'],	
			);
		}
		return array("total" => $total, 
					 "rows"  => $res);
	}	
	
	/*
		DisplayStatus 
		@param int $status, 
		@param array $payload 
		
		@return string (
			'A' - door 1 open
			'B' - door 1 closed
			'C' - door 2 open
			'D' - door 2 closed
			'O' - system message
			'X' - alert	
		)
	*/
	public function DisplayStatus( $status, $payload )
	{
		if ( $payload['issys'] ) {
			$res = 'O';
		}
		else {
			if ($payload['pir'] == 1) {
				$res = 'P';
			}
			else {		
				$res = ($payload['door1'] == 0 ?'A' :'B').($payload['door2'] == 0 ?'C' :'D');
			}
		}
				
		return '  '.$res.( ($status == 3) ? 'X' : '' );
	}
	
	public function DisplayStatus2( $payload )
	{
		return $payload['payload'].':'.$payload['var'].':sys:'.$payload['issys'].':'.$payload['bin3'].':'.($payload['pir'] == 1 ?'pir':'');
	}	


	
	/**
	* Get all Alerts for user
	* @param int userid
	* @param String dt1
	* @param String dt2
	*
	* @return $res array
	*/
	public function getAlerts($userid, $dt1, $dt2)
	{	

		$stmt = $this->getEntityManager()->getConnection()->prepare('SELECT * FROM getAlerts(:userid, :dt1, :dt2) LIMIT 10 ');
		$stmt->bindValue('userid', $userid);			
		$stmt->bindValue('dt1', $dt1);
		$stmt->bindValue('dt2', $dt2);
																					  
		$stmt->execute();
		$rows = $stmt->fetchAll();			
		 
		$res = array();
		$stats = Common::getUnitStatuses();	
		
		$x_max = -180;
		$y_max = -180;
		$x_min = 180;
		$y_min = 180;
			
		foreach ($rows as $row) {

			$y_max = ($y_max < (float)$row['latitude']) ? (float)$row['latitude']: $y_max;
			$y_min = ($y_min > (float)$row['latitude']) ? (float)$row['latitude']: $y_min;

			$x_max = ($x_max < (float)$row['longitude']) ? (float)$row['longitude']: $x_max;
			$x_min = ($x_min > (float)$row['longitude']) ? (float)$row['longitude']: $x_min;
				
			$res[] = array(
				'id' 			=> (int)$row['id'],		
				'ownid' 		=> (int)$row['ownid'],	
				'stationid' 	=> (int)$row['stationid'],				
				'sname' 		=> $row['sname'],	
				'esn' 			=> $row['esn'],
				'ibuttonid' 	=> (int)$row['ibuttonid'],				
				'iname' 		=> $row['iname'],	
				'keyholderid'	=> (int)$row['keyholderid'],				
				'kname' 		=> $row['kname'],		
				'longitude'		=> (float)$row['longitude'],				
				'latitude' 		=> (float)$row['latitude'],					
				'dt' 			=> $row['dt'],				
				'charge'		=> (int)$row['charge'],
				'status' 		=> (int)$row['status'],	
				'statname' 		=> $stats[ $row['status'] ],
				'edata' 		=> $row['edata'],	
				'location' 		=> $row['location'],	
			);
		}
		
		return array('rows' => $res,
					 'y_coor' => ($y_max+$y_min)/2,
					 'x_coor' => ($x_max+$x_min)/2,
					 'zoom'   => Common::detectZoom(MAX( ABS($x_max - $x_min), ABS($y_max - $y_min)))
					 );
	}		
	
	/**
	* Get all Alerts for user
	* @param int userid
	* @param String dt1
	* @param String dt2
	*
	* @return $res array
	*/
	public function getAlertsForContact($contactid, $dt1, $dt2)
	{	

		$stmt = $this->getEntityManager()->getConnection()->prepare('SELECT * FROM getAlertsForContact(:contactid, :dt1, :dt2)');
		$stmt->bindValue('contactid', $contactid);			
		$stmt->bindValue('dt1', $dt1);
		$stmt->bindValue('dt2', $dt2);
																					  
		$stmt->execute();
		$rows = $stmt->fetchAll();			
		 
		$res = array();
		$stats = Common::getUnitStatuses();	
		
		$x_max = -180;
		$y_max = -180;
		$x_min = 180;
		$y_min = 180;
			
		foreach ($rows as $row) {

			$y_max = ($y_max < (float)$row['latitude']) ? (float)$row['latitude']: $y_max;
			$y_min = ($y_min > (float)$row['latitude']) ? (float)$row['latitude']: $y_min;

			$x_max = ($x_max < (float)$row['longitude']) ? (float)$row['longitude']: $x_max;
			$x_min = ($x_min > (float)$row['longitude']) ? (float)$row['longitude']: $x_min;
				
			$res[] = array(
				'id' 			=> (int)$row['id'],		
				'ownid' 		=> (int)$row['ownid'],	
				'stationid' 	=> (int)$row['stationid'],				
				'sname' 		=> $row['sname'],	
				'esn' 			=> $row['esn'],
				'ibuttonid' 	=> (int)$row['ibuttonid'],				
				'iname' 		=> $row['iname'],	
				'keyholderid'	=> (int)$row['keyholderid'],				
				'kname' 		=> $row['kname'],		
				'longitude'		=> (float)$row['longitude'],				
				'latitude' 		=> (float)$row['latitude'],					
				'dt' 			=> $row['dt'],				
				'charge'		=> (int)$row['charge'],
				'status' 		=> (int)$row['status'],	
				'statname' 		=> $stats[ $row['status'] ],
				'edata' 		=> $row['edata'],	
				'location' 		=> $row['location'],	
			);
		}
		
		return array('rows' => $res,
					 'y_coor' => ($y_max+$y_min)/2,
					 'x_coor' => ($x_max+$x_min)/2,
					 'zoom'   => Common::detectZoom(MAX( ABS($x_max - $x_min), ABS($y_max - $y_min)))
					 );
	}			

	/**
	* Get notifiers for event
	* @param int eventid
	* @param int $ownid
	* @return $res array
	*/	
	public function getNotifies( $eventid, $ownid ) 
	{
		$event = $this->find( $eventid );
		$res = array();		
		if ($event) {
		
			$alertsys = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Alertstations')->getLocation( $event->getStationid() );
			$rep_nf = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Notifications');
			$contacts = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Alertcontacts')->getContacts($alertsys['id'], $ownid);		
			foreach($contacts as $cont) {
		
				$phone = $rep_nf->checkNotify((int)$eventid, (int)$cont['id'], 1);			
				$sms   = $rep_nf->checkNotify((int)$eventid, (int)$cont['id'], 2);
				$email = $rep_nf->checkNotify((int)$eventid, (int)$cont['id'], 3);
				$close = $rep_nf->checkNotify((int)$eventid, (int)$cont['id'], 4);
		
				$res[] = array(
					'id'    => (int)$cont['id'],
					'name'  => $cont['lname'].' '.$cont['fname'],
					'phone' => $cont['phone1'],
					'email' => (int)$email['result'],
					'sms'   => (int)$sms['result'],
					'close' => (int)$close['result'],
					'tel'   => (int)$phone['result'],
					'dt'    => $phone['dt'],	
				);
			}
		}
		return $res;
	}	
	
	/**
	* Export to excel file
	*
	* @return $res array
	*/
	public function exportList($phpExcelObject, $userid, $stationid, $ibuttonid, $keyholderid, $status, $dt1, $dt2)
	{
		$phpExcelObject->getProperties()->setCreator("SosMon datafile")
			->setTitle("MS Excel spreadsheet");		
	
		$sheet = $phpExcelObject->setActiveSheetIndex(0);
		$nr = 2;
		$sheet->setCellValue('B'.$nr, 'Events list');
		$nr++;			

		$styleArray = array(
			'borders' => array(
				'allborders' => array(
					'style' => 'double',
				)
			)
		);

		$sheet->getStyle('B'.$nr.':H'.$nr)->applyFromArray($styleArray);
		unset($styleArray);
		
		$sheet->setCellValue('B'.$nr, 'Nr')
			->setCellValue('C'.$nr, 'Time')
			->setCellValue('D'.$nr, 'Station')
			->setCellValue('E'.$nr, 'ESN')
			->setCellValue('F'.$nr, 'Status')
			->setCellValue('G'.$nr, 'IButton')
			->setCellValue('H'.$nr, 'Key holder');				
		$nr++;  
		
		$sheet->getColumnDimension('B')->setWidth(10);	
		$sheet->getColumnDimension('C')->setWidth(20);	
		$sheet->getColumnDimension('D')->setWidth(20);	
		$sheet->getColumnDimension('E')->setWidth(20);
		$sheet->getColumnDimension('F')->setWidth(20);	
		$sheet->getColumnDimension('G')->setWidth(20);		
		$sheet->getColumnDimension('H')->setWidth(20);
				
		$cnt= 0;
		$list = $this->getList( 0, 0, $userid, $stationid, $ibuttonid, $keyholderid, $status, $dt1, $dt2);			   
		foreach ($list['rows'] as $row) {
			$cnt++;
			$sheet->setCellValue('B'.$nr, $cnt)
				->setCellValue('C'.$nr, $row['dt'])
				->setCellValue('D'.$nr, $row['sname'])
				->setCellValue('E'.$nr, $row['esn'])
				->setCellValue('F'.$nr, $row['statname'])
				->setCellValue('G'.$nr, $row['iname'])
				->setCellValue('H'.$nr, $row['kname']);
			$nr++;
		}		
		    	   
		$phpExcelObject->getActiveSheet()->setTitle('Events');
       // Set active sheet index to the first sheet, so Excel opens this as the first sheet
		$phpExcelObject->setActiveSheetIndex(0);		
		
		return $phpExcelObject;
	}	
	
	/**
	* Get entity SosMon station
	*
	* @param int $id
	* @param int $ownid	
	* @return entity $entity
	*/			
	public function findEntity($id, $ownid)
	{
		$entity = $this->find($id);	
		if (!$entity) {
			$entity = new Event(); 
			$entity->setOwnid( $ownid );
		}
				
		return array(			
				'id' 			=> (int)$entity->getId(),		
				'ownid' 		=> (int)$entity->getOwnid(),	
				'stationid'	 	=> $entity->getStationid(),
				'ibuttonid' 	=> $entity->getIbuttonid(),
				'keyholderid' 	=> $entity->getKeyholderid(),
				'status'		=> (int)$entity->getStatus(),
				'longitude' 	=> (float)$entity->getLongitude(),		
				'latitude' 		=> (float)$entity->getLatitude(),
				'charge'		=> (int)$entity->getCharge(),
				'rawid'			=> (int)$entity->getRawid(),
				'response'      => (int)$entity->getResponse(),
				'dt' 			=> Common::fmtDateTime($entity->getDt()),				
				'edata' 		=> $entity->getEdata(),			
					);					
	}
	
	/**
	* Get entity SosMon station
	*
	* @param int $eventid
	* @param int $stationid	
	* @return string
	*/			
	public function findPreviousEdata($eventid, $stationid)
	{
		$stmt = $this->getEntityManager()->getConnection()->prepare('SELECT edata FROM events WHERE stationid = :stationid and id < :eventid ORDER BY id DESC LIMIT 1 ');
		$stmt->bindValue('stationid', $stationid);			
		$stmt->bindValue('eventid', $eventid);
																					  
		$stmt->execute();
		$rows = $stmt->fetchAll();		
		
		return (count($rows) == 1 ? $rows[0]['edata'] : '0x000000000000000000');
	}	
	
	
	
	
	/**
	* Get entity description
	*
	* @param int $id
	* @param int $ownid	
	* @return entity $entity
	*/			
	public function eventDescription($id)
	{	
		$event = $this->find($id);
		if ($event) {	
			$ibutton   = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Ibutton')->findEntity($event->getIbuttonid(), 0);
			$keyholder = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Keyholder')->findEntity($event->getKeyholderid(), 0);
		
			$stat =  Common::getUnitStatuses();			
			return Common::fmtDateTime($event->getDt()).' '.$stat[ $event->getStatus() ].' '.$ibutton['key'].' '.$keyholder['lname'].' '.$keyholder['fname'];		
		}
		else {
			return '< None >';
		}
	}	
	
	/**
	* Save Station
	*
	* @param array $var  
	* @return entity id
	*/
	public function saveEntity( $var )
	{
		$entity = $this->find((int)$var['id']);	
		if (!$entity) {
			$entity = new Event();
		}
		
		$entity->setOwnid( (int)$var['ownid'] );
		$entity->setRawid( (int)$var['rawid'] );
		$entity->setStationid( (int)$var['stationid'] );
		$entity->setIbuttonid( (int)$var['ibuttonid'] );
		$entity->setKeyholderid( (int)$var['keyholderid'] );
		$entity->setStatus( (int)$var['status'] );
		$entity->setLongitude( $var['longitude'] );
		$entity->setLatitude( $var['latitude'] );
		$entity->setCharge( (int)$var['charge'] );
		$entity->setEdata( $var['edata'] );

		$entity->setDt( DateTime::createFromFormat('m/d/Y H:i:s', $var['dt']) );
					
		$em = $this->getEntityManager();				
		$em->persist($entity);
		$em->flush();
		return $entity->getId();			
	}
	
	/**
	* Save Response for event
	*
	* @param int $id
	* @param int $response
	* @return entity id
	*/
	public function saveResponse( $id, $contact, $response )
	{
		$entity = $this->find( $id );	
		if ($entity) {
			$now = new DateTime("now");
			
			$entity->setResponse( (int) $response )
				   ->setResponse_user((int)$contact)
				   ->setResponse_dt( $now );
			$em = $this->getEntityManager();				
			$em->persist($entity);
			$em->flush();			
		}		
		return $id;			
	}	
	
	/**
	* Delete event
	*
	* @param int $id
	* @return Bool
	**/
	public function deleteEntity( $id )
	{	
		$res = false;
		$entity = $this->find( $id );	
		if ($entity) {
			$em = $this->getEntityManager();		
			$em->remove( $entity );
			$em->flush();		
			$res = true;
		} 
					 
		return $res;
	}	
	
	/**
	* Array of markers for events. 
	* Used when displaying track markers for specified station
	* @param int $ownid
	* @param int $systemid	
	* @param int $stationid
	* @param int $ibuttonid
	* @param int $keyholderid
	* @param int $status
	* @param String $dt1
	* @param String $dt2	
	* @param int $xcoord
	* @param int $ycoord	
	* @return array 
	**/	
	public function getMarkersForEvents($userid, $systemid, $stationid, $ibuttonid, $keyholderid, $status, $dt1, $dt2, $xcoord, $ycoord)
	{	
	
		$x_max = $xcoord;
		$y_max = $ycoord;
		$x_min = $xcoord;
		$y_min = $ycoord;
		
		$markers = array('x_center' => $xcoord,
						 'y_center' => $ycoord,
						 'zoom' 	=> 10,
						 'items' 	=> array());
					       
		$rows = $this->getList(0, 0, $userid, $systemid, $stationid, $ibuttonid, $keyholderid, $status, $dt1, $dt2);	
		
		if (count($rows["rows"]) > 0) {
			$x_max = -180;
			$y_max = -180;
			$x_min = 180;
			$y_min = 180;		

			$aIcon = Common::getMarkerType();
			$i = 0;	 
			foreach ($rows["rows"] as $row) {
		
				$y_max = ($y_max < (float)$row['latitude']) ? (float)$row['latitude']: $y_max;
				$y_min = ($y_min > (float)$row['latitude']) ? (float)$row['latitude']: $y_min;

				$x_max = ($x_max < (float)$row['longitude']) ? (float)$row['longitude']: $x_max;
				$x_min = ($x_min > (float)$row['longitude']) ? (float)$row['longitude']: $x_min;
			
				$markers['items'][] = array('x_c'   => (float)$row['longitude'],
											'y_c'   => (float)$row['latitude'],
											'text'  => $row['dt'],
											'state' => ($i == 0 )? $aIcon[ (int)$row['status'] ] : $aIcon[ (int)$row['status'] + 4] , 
											'id'    => (int)$row['id'],
											'entity_kind'    => 2,											
											);
				$i++;
			}
		}
		
		$markers['x_center'] = ($x_max+$x_min)/2;
		$markers['y_center'] = ($y_max+$y_min)/2;
		$markers['zoom']     =  Common::detectZoom(MAX( ABS($x_max - $x_min), ABS($y_max - $y_min)));
		
		return $markers;		
	}	
}