<?php
namespace Acme\SosMonBundle\Entity;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\NoResultException;

class IbuttonRepository extends EntityRepository 
{

	/**
	* Get all Ibuttons
	* @param int $ownid
	* @param int $opt
	*
	* @return $res array
	*/
	public function getList( $ownid, $opt, $key = '' )
	{	
	
		$keys = $this->findBy(array('ownid' => $ownid, 'deleted' => 0), array('key' => 'ASC'));
						 
		$res = array();
		if ($opt == 1) {
			$res[] = array(
				'id' 		=> 0,			
				'key' 		=> '< ALL >',	
				'keyholderid'=> 0,
				'name' => '',
				'crc'  => '',				
				'active' => 0,				
			);		
		}
		
		if ($opt == 2) {
			$res[] = array(
				'id' 		=> 0,			
				'key' 		=> '< NONE >',	
				'keyholderid'=> 0,
				'name' => '',
				'crc'  => '',
				'active' => 0,
			);		
		}		
		
		foreach ($keys as $ikey) {
			$p1 = (strlen($key) == 0) ? true : strpos( $ikey->getKey().$ikey->getCrc(), $key );
			
			
			if ($p1 !== false) {
				$keyholder = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Keyholder')->getIbuttonOwner($ikey->getId());
				$res[] = array(
					'id' 		 => $ikey->getId(),		
					'key' 		 => $ikey->getKey(),	
					'crc'        => $ikey->getCrc(),	
					'keyholderid'=> $keyholder['id'],	
					'name' 		 => $keyholder['name'],	
					'active' 	 => $ikey->getActive(),
				);			
			}
			
		}
		return $res;
	}	

	/**
	* Get all ibuttons
	*
	* @return $excelObject
	*/
	public function exportList($phpExcelObject, $ownid)
	{
		$phpExcelObject->getProperties()->setCreator("SosMon datafile")
			->setTitle("MS Excel spreadsheet");		
	
		$sheet = $phpExcelObject->setActiveSheetIndex(0);	

		$styleArray = array(
			'borders' => array(
				'allborders' => array(
					'style' => 'double',
				)
			)
		);
		
		$styleArray0 = array(
			'borders' => array(
				'allborders' => array(
					'style' => 'none',
				)
			)
		);
		
		$nr = 2;
		$sheet->setCellValue('A'.$nr, 'iButtons list')
			->mergeCells('A2:B2');
		$nr++;			

		$sheet->getStyle('A1:E4')->applyFromArray($styleArray0);		

		$sheet->getStyle('A'.$nr.':E'.$nr)->applyFromArray($styleArray);
		unset($styleArray);
		
		$sheet->setCellValue('A'.$nr, 'Nr')
			->setCellValue('B'.$nr, 'Key')
			->setCellValue('C'.$nr, 'CRC')
			->setCellValue('D'.$nr, 'Assigned')
			->setCellValue('E'.$nr, 'Status');
			
		$nr++;  
		
		$sheet->getColumnDimension('A')->setWidth(10);	
		$sheet->getColumnDimension('B')->setWidth(20);	
		$sheet->getColumnDimension('C')->setWidth(20);	
		$sheet->getColumnDimension('D')->setWidth(20);		
		$sheet->getColumnDimension('E')->setWidth(10);
				
		$cnt= 0;
		$list = $this->getList( $ownid, 0 );			   
		foreach ($list as $row) {
			$cnt++;
			$sheet->setCellValue('A'.$nr, $cnt)
				->setCellValue('B'.$nr, ' '.$row['key'])
				->setCellValue('C'.$nr, ' '.$row['crc'])
				->setCellValue('D'.$nr, $row['name'])
				->setCellValue('E'.$nr, ($row['active']==1?'ON':'OFF'));
			$nr++;
		}		
		    	   
		$phpExcelObject->getActiveSheet()->setTitle('iButtons');
       // Set active sheet index to the first sheet, so Excel opens this as the first sheet
		$phpExcelObject->setActiveSheetIndex(0);		
		
		return $phpExcelObject;
	}	
	
	/**
	* Get available iButtons
	* @param int $ownid
	* @param int $keyholderid
	*
	* @return $res array
	*/
	public function getAvailableIbuttons( $ownid, $keyholderid )
	{	
		$res = array();
		$res[ 0 ] = 'NONE';
				
		$keys = $this->findBy(array('ownid' => $ownid, 'deleted' => 0), array('key' => 'ASC'));
		
		foreach($keys as $key) {
			$id = $key->getId();
			$row = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Keyholder')->getIbuttonOwner($id);
			if (( (int)$row['id'] == 0) or ((int)$row['id'] == $keyholderid )){
				$res[ $key->getId() ] = $key->getKey();
			}
		}

		return $res;
	}
		
	/**
	* Get entity SosMon ibutton
	*
	* @param int $id
	* @param int $ownid
	* @return entity $entity
	*/			
	public function findEntity($id, $ownid)
	{
		$entity = $this->find($id);	
		if (!$entity) {
			$entity = new Ibutton(); 
			$entity->setOwnid( $ownid );			
		}
		
		return array(	'id' 		=> (int)$entity->getId(),			
						'ownid' 	=> (int)$entity->getOwnid(),			
						'key' 		=> $entity->getKey(),
						'crc' 		=> $entity->getCrc(),
						'active' 	=> (int)$entity->getActive(),
					);		
	}
	
	/**
	* Save Ibutton
	*
	* @param array $var  
	* @return entity id
	*/
	public function saveEntity( $var )
	{
		$entity = $this->find((int)$var['id']);	
		if (!$entity) {
			$entity = new Ibutton();
		}
	
		$entity->setOwnid(   (int)$var['ownid'] ); 
		$entity->setKey(     $var['key'] );		
		$entity->setCrc(     $var['crc'] );
		$entity->setActive(  (int)$var['active']);
					
		$em = $this->getEntityManager();				
		$em->persist($entity);
		$em->flush();	
		$id = $entity->getId();	
		
		$this->getEntityManager()->getRepository('AcmeSosMonBundle:Keyholder')->assignButton((int)$var['keyholderid'], $id);

		return $id;			
	}
	
	/**
	* checkEntity
	*
	* @param array $var
	* @return Bool
	**/
	public function checkEntity( $var )
	{
		$res = array('success' => true,
					 'message' => '' );
					 
		$keys = $this->findBy(array('crc' => $var['crc'], 'deleted' => 0));
		foreach( $keys as $item) {
			if ($item->getId() != (int)$var['id']) {
				$res['success'] = false;
				$res['message'] = 'Key with CRC '.$var['crc'].' already exists';
			}
		}
		
		return $res;
	}	
		
	/**
	* Ibuttons set active status
	*
	* @param int $id
	* @param int $active
	* @return entity id
	*/
	public function activeEntity( $id)
	{	
	
		$entity = $this->find( $id );	
		if ($entity) {
			$active = (($entity->getActive() == 0 ) ? 1 : 0);
			$entity->setActive(  $active );
			$em = $this->getEntityManager();	
			$em->persist($entity);
			$em->flush();			
		}
		 
		return $id;
	}	
	
	/**
	* Delete ibutton
	*
	* @param int $id
	* @return 1
	**/
	public function deleteEntity( $id )
	{	
		$entity = $this->find( $id );	
		if ($entity) {
			$em = $this->getEntityManager();						
			$entity->setDeleted( 1 ); 
			$em->persist($entity);				
			$em->flush();		
	    }		 								 
		return $entity;
	}	
	

	
}