<?php
namespace Acme\SosMonBundle\Entity;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\NoResultException;

class KeyholderRepository extends EntityRepository 
{

	/**
	* Get all Keyholders
	*
	* @return $res array
	*/
	public function getList($ownid, $opt)
	{	
		
		$holders = $this->findBy(array('ownid' => $ownid, 'deleted' => 0), array('lname' => 'ASC', 'fname' => 'ASC'));				 
		$res = array();

		if ($opt == 1) {
			$res[] = array(
				'id' 		=> 0,			
				'name' 		=> '< ALL >',	
			);		
		}
		
		if ($opt == 2) {
			$res[] = array(
				'id' 		=> 0,			
				'name' 		=> '< NONE >',	
			);		
		}		
		
		foreach ($holders as $holder) {
			$key = '';
			if ($holder->getIbuttonid() > 0) {
				$ibutton = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Ibutton')->findEntity( $holder->getIbuttonid(), $ownid);
				$key = $ibutton['key'];
			}
			
			$res[] = array(
				'id' 		=> $holder->getId(),	
				'name'      => $holder->getLname().' '.$holder->getFname(),
				'fname' 	=> $holder->getFname(),	
				'lname' 	=> $holder->getLname(),				
				'phone' 	=> $holder->getPhone(),					
				'ibutton' 	=> $key,
				'ibuttonid'	=> $holder->getIbuttonid(),	
			);
		}
		return $res;
	}	

	/**
	* Get available iButtons
	*
	* @return $res array
	*/
	public function getAvailableKeyholders( $ownid, $ibuttonid )
	{	
		$holders = $this->findBy(array('ownid' => $ownid, 'deleted' => 0), array('lname' => 'ASC', 'fname' => 'ASC'));
		
		$res = array();
		$res[ 0 ] = 'NONE';
		
		foreach ($holders as $row) {
			if (($row->getIbuttonid() == 0) || ($row->getIbuttonid() == $ibuttonid))
			{
				$res[ $row->getId() ] = $row->getLname().' '.$row->getFname();
			}
		}
		return $res;
	}
	
	public function getIbuttonOwner( $ibuttonid )
	{	
		$entity = $this->findOneBy(array('ibuttonid' => $ibuttonid, 'deleted' => 0) );
		
		$res = array( 'id' => 0,
					  'name'=> '',
					);
					
		if ($entity) {
			$res['id'] = $entity->getId();
			$res['name'] = $entity->getLname().' '.$entity->getFname();
		}
		return $res;
	}
	
	public function freeKey( $ibuttonid )
	{	
		$em = $this->getEntityManager();
		
		$other = $this->findByIbuttonid( $ibuttonid );
		foreach($other as $entity) {
			$entity->setIbuttonid( 0 );
			$em->persist($entity);		
		}
		$em->flush();
		
		return 1;
	}	

	public function assignButton($keyholder, $key) 
	{
		$em = $this->getEntityManager();	

		$entity = $this->find($keyholder);
		if ($entity) {
			$this->freeKey( $key );
		
			$entity->setIbuttonid( $key );	
			$em->persist($entity);
			$em->flush();	
		}
		return 1;
	}
	

	
	/**
	* Get available Keyholders for transfer
	*
	* @return $res array
	*/
	public function getTransferList( $ownid )
	{	
		$holders = $this->findBy(array('ownid'=>$ownid, 'deleted' => 0), array('lname' => 'ASC'));
				 
		$res = array();
		$res[] = array(
			'id' 		 => 0,		
			'name' 		 => 'NONE',	
		);
		
		foreach ($holders as $holder) {
		
			$key = '';
			if ($holder->getIbuttonid() > 0) {
				$ibutton = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Ibutton')->findEntity( $holder->getIbuttonid(), $ownid);
				$key = '('.$ibutton['key'].')';
			}
			
			$res[] = array(
				'id' 		 => $holder->getId(),		
				'name' 		 => $holder->getLname().' '.$holder->getFname().$key,
			);
		}
		return $res;
	}	
	
	/**
	* Get entity SosMon keyholder
	*
	* @param int $id
	* @param int $ownid	
	* @return entity $entity
	*/			
	public function findEntity($id, $ownid)
	{
		$entity = $this->find($id);	
		if (!$entity) {
			$entity = new Keyholder(); 
			$entity->setOwnid( $ownid );
		}
		
		return array(	'id'		=> (int)$entity->getId(),			
						'ownid' 	=> (int)$entity->getOwnid(),			
						'ibuttonid'	=> (int)$entity->getIbuttonid(),		
						'lname' 	=> $entity->getLname(),
						'fname' 	=> $entity->getFname(),
						'phone' 	=> $entity->getPhone(),	
					);		
	}
	
	/**
	* Save Keyholder
	*
	* @param array $var  
	* @return entity id
	*/
	public function saveEntity( $var )
	{
		$entity = $this->find((int)$var['id']);	
		if (!$entity) {
			$entity = new Keyholder();
		}
		try {
			$entity->setIbuttonid((int)$var['ibuttonid']);		
			$entity->setOwnid(   (int)$var['ownid']);
			$entity->setFname(   $var['fname']);			
			$entity->setLname(   $var['lname']);	
			$entity->setPhone(   $var['phone']);		
					
			$em = $this->getEntityManager();				
			$em->persist($entity);
			$em->flush();
			$id = $entity->getId();
		}
		catch(Exception $e) {
			$id = -1;
		}
		return $id;			
	}
	
	/**
	* Delete customer
	*
	* @param int $id
	* @return 1
	**/
	public function deleteEntity( $id )
	{	
		$entity = $this->find( $id );	
		if ($entity) {
			$em = $this->getEntityManager();						
			$entity->setDeleted( 1 ); 
			$em->persist($entity);				
			$em->flush();		
	    }		 								 
		return $entity;
	}	
		
}