<?php
namespace Acme\SosMonBundle\Entity;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\NoResultException;
use \DateTime;
use Acme\SosMonBundle\Model\Common;

class KeystationsRepository extends EntityRepository 
{
/*
  getKeystations
  return keys for station OR stations for key OR assignment {key, station}
  
  @param int $key
  @param int $stationid
  @return array
*/	
	public function getKeystations($key, $stationid)
	{
		$res = array();
		$em = $this->getEntityManager();
		if (($key > 0) && ($stationid == 0)) {
			$rows = $this->findBy(array('ibuttonid' => $key));
			$ibutton = $em->getRepository('AcmeSosMonBundle:Ibutton')->findEntity($key, 0);

			foreach ($rows as $row) {
				$station = $em->getRepository('AcmeSosMonBundle:Station')->findEntity($row->getStationid(), 0);
				
				$res[] = array( 'stationid' => $row->getStationid(),
								'ibuttonid' => $row->getIbuttonid(),
								'name' 		=> $station['name'].'('.$station['esn'].')',
								'key'  		=> $ibutton['key'],
								);
			}
		}
		
		if (($stationid > 0) && ($key == 0)) {
			$rows = $this->findBy(array('stationid' => $stationid));
			$station= $em->getRepository('AcmeSosMonBundle:Station')->findEntity($stationid, 0);

			foreach ($rows as $row) {
				$ibutton = $em->getRepository('AcmeSosMonBundle:Ibutton')->findEntity($row->getIbuttonid(), 0);
			
				$res[] = array( 'stationid' => $row->getStationid(),
							'ibuttonid' => $row->getIbuttonid(),
							'name' 		=> $station['name'].'('.$station['esn'].')',
							'key'  		=> $ibutton['key'],
							);
			}			
		}
		
		if (($stationid > 0) && ($key > 0)) {
			$rows = $this->findBy(array('stationid' => $stationid, 
										'ibuttonid' => $key	));
										
			$station= $em->getRepository('AcmeSosMonBundle:Station')->findEntity($stationid, 0);
			$ibutton = $em->getRepository('AcmeSosMonBundle:Ibutton')->findEntity($key, 0);
			
			foreach ($rows as $row) {
				$res[] = array( 'stationid' => $stationid,
								'ibuttonid' => $key,
								'name' 		=> $station['name'].'('.$station['esn'].')',
								'key'  		=> $ibutton['key'],
							);
			}			
		}		
		
		return $res;
	}
	
/*
  getCrcstations
  is available or not
  
  @param string $crc
  @param int $stationid
  @return bool
*/	
	public function getCrcstations($crc, $stationid)
	{
		$res = false;
		$em = $this->getEntityManager();
		$ibutton = $em->getRepository('AcmeSosMonBundle:Ibutton')->findOneByCrc($crc);
			
		if (($stationid > 0) && (is_object($ibutton ))) {
			$rows = $this->findBy(array('stationid' => $stationid, 
										'ibuttonid' => $ibutton->getId()	));

										
			if (count( $rows) > 0) {
				$res = true;
			}
		}		
		
		return $res;
	}	
	
	
	
	/**
	* Assign keys to station
	*
	* @return $res array
	*/
	public function assignKeys( $id, $keys )
	{	
		$this->deleteForStation( $id );
		
		foreach($keys as $row) {
			if ((int)$row > 0) {
				$this->saveEntity( array (  'stationid' => $id,
											'ibuttonid' => $row) );
			}
		}	
		return 1;
	}
	
	/**
	* Assign stations to iButton
	*
	* @return $res array
	*/
	public function assignStations( $id, $keys )
	{	
		$this->deleteForIbutton( $id );
		
		foreach($keys as $row) {
			if ((int)$row > 0) {		
				$this->saveEntity( array (  'stationid' => $row,
											'ibuttonid' => $id) );
			}
		}	
		return 1;
	}	

	/**
	* Save Keystation
	*
	* @param array $var  

	* @return entity id
	*/	
	public function saveEntity( $var )
	{
		$entity = new Keystations();
		
		$entity->setStationid( (int)$var['stationid'] );
		$entity->setIbuttonid( (int)$var['ibuttonid'] );
					
		$em = $this->getEntityManager();				
		$em->persist($entity);
		$em->flush();
		return $entity->getId();			
	}

	/**
	* Delete for station
	*
	* @param int $id
	* @return 1
	**/
	public function deleteForStation( $id )
	{		
		$this->getEntityManager()->getConnection()->executeUpdate('delete from keystations where stationid = :id', 
			array('id' => $id)
		);			 
		return 1;
	}
	
	/**
	* Delete for iButton
	*
	* @param int $id
	* @return 1
	**/
	public function deleteForIbutton( $id )
	{		
		$this->getEntityManager()->getConnection()->executeUpdate('delete from keystations where ibuttonid = :id', 
			array('id' => $id)
		);			 
		return 1;
	}	
}