<?php
namespace Acme\SosMonBundle\Entity;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\NoResultException;
use \DateTime;
use Acme\SosMonBundle\Model\Common;

class NotificationsRepository extends EntityRepository 
{

	/**
	* Predefined array of unit types
	*
	* @return array access types
	**/
	public function getNotificationTypes()
	{	
		return array(  	1 => 'Phone',
						2 => 'Sms',
						3 => 'Email',
					);
	}
	
	public function checkNotify($eventid, $contactid, $ntype) 
	{
		$item = $this->findOneBy(array('eventid'   => $eventid,
									'contactid' => $contactid,
									'ntype'     => $ntype));
									
		$res = 2;
		$dt  = '';
		if ($item) {
			$res = $item->getSuccess();
			$dt  = Common::fmtDateTime($item->getDt());
		}
		
		return array('result' => $res,
					 'dt'     => $dt);
	}
	
	/*
	** NotificationsForEvent
	** @param int $eventid
	**
	** @return array
	*/
	
	public function NotificationsForEvent($eventid) 
	{
		$stmt = $this->getEntityManager()->getConnection()->prepare("SELECT n.*, c.lname||' '||c.fname as cname, CASE WHEN success = 1 THEN 'success' ELSE 'fail' END as result FROM  notifications n
																	 LEFT JOIN contacts c ON c.id = n.contactid
																	 WHERE eventid = :eventid
																	 ORDER BY dt DESC");
		$stmt->bindValue('eventid', $eventid);		
		$stmt->execute();
		return $stmt->fetchAll();
	}	
	
	
	/**
	$this->get('CallRegistration')->regPhoneNotification(23, 12, 1);
	
	* Save Notifications
	*
	* @param array $var  

	* @return entity id
	*/
	public function saveEntity( $var )
	{
		$entity = $this->findOneBy(array('eventid'   => (int) $var['eventid'],
										 'contactid' => (int) $var['contactid'],
										 'ntype'     => (int) $var['ntype']));
		if (!$entity) {						
			$entity = new Notifications();
			$entity->setContactid(    $var['contactid'] );
			$entity->setEventid(      $var['eventid'] );
			$entity->setNtype(        $var['ntype'] );			
		}
		$entity->setSuccess(      $var['success'] );
		$entity->setDt( new DateTime);
					
		$em = $this->getEntityManager();				
		$em->persist($entity);
		$em->flush();
		return $entity->getId();			
	}
	
	/**
	*  Notifications summary
	*  @param @ownid
	*  @param @locationid
	*  @param @dt1
	*  @param @dt2
	*
	*  @return 	array('ping_msg'  => 1,
					  'stat_msg'  => 2,
					  'alarm_msg' => 3,
					  'door_msg'  => 4,
					  'other_msg' => 5,
					  'total_msg' => 123,
					  'f_phone'   => 6,
					  's_phone'   => 7,
					  'total_phone' => 52,
					  'sms_sent'  => 8,
					  'email_sent'=> 9,
				);
	*/
	public function getSummary($params) 
	{
		$stmt = $this->getEntityManager()->getConnection()->prepare('SELECT * FROM getSummary(:ownid, :locationid, :dt1, :dt2)');
		$stmt->bindValue('ownid', $params['ownid']);		
		$stmt->bindValue('locationid', $params['locationid']);	
		$stmt->bindValue('dt1', $params['dt1']);
		$stmt->bindValue('dt2', $params['dt2']);
		$stmt->execute();
		$rows = current( $stmt->fetchAll() );	
		
		return $rows;		
	}
	
	public function exportSummary($phpExcelObject,  $params) 
	{
		$res = $this->getSummary($params);
		
		$phpExcelObject->getProperties()->setCreator("SosMon datafile")
			->setTitle("MS Excel spreadsheet");		
	
		$sheet = $phpExcelObject->setActiveSheetIndex(0);

		$styleArray = array(
			'borders' => array(
				'inside' => array(
					'style' => 'thin',
				),		
				'outline' => array(
					'style' => 'double',
				)
			)
		);
		$style_0 = array(
			'font' => array( 'bold' => true, 'size' => 16),
		);
		
		$style_1 = array(
			'font' => array( 'size' => 12),
		);		
		
		$style_2 = array(
			'font' => array('bold' => true, 'size' => 12),
			'alignment' => array('horizontal' => 1,),
		);
		
		$styleArray0 = array(
			'borders' => array(
				'allborders' => array(
					'style' => 'none',
				)
			)
		);
		
		$sheet->getStyle('A1:B6')->applyFromArray($styleArray0);		
	
		$sheet->getStyle('A7:B20')->applyFromArray($styleArray);

		$sheet->setCellValue('A2', 'Usage summary');
	
		if ($params['ownid'] == -1) {
			$entity = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Alertsystem')->findEntity($params['locationid'], 0);
			$sheet->setCellValue('A4', 'Location: '.($params['locationid'] == 0 ? 'ALL' : $entity['name']));
		}
		else {
			$entity = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Customer')->findEntity($params['ownid'], 0);		
			$sheet->setCellValue('A4', 'Company: '.($params['ownid'] == 0 ? 'ALL' : $entity['name']));
		}
		$sheet->setCellValue('A5', 'Period :'. Common::zfdate($params['dt1'], 'sql', 'us') .
								   ' - '.Common::zfdate($params['dt2'], 'sql', 'us') );
		$sheet->mergeCells('A5:B5');

		$sheet->setCellValue('A7','Message Usage Totals')->mergeCells('A7:B7')
			->setCellValue('A8','Ping messages')
			->setCellValue('B8', $res['ping_msg'])
			->setCellValue('A9','Statistical messages')
			->setCellValue('B9', $res['stat_msg'])
			->setCellValue('A10','Door open/close messages')
			->setCellValue('B10', $res['door_msg'])
			->setCellValue('A11','Alarm messages')
			->setCellValue('B11', $res['alarm_msg'])
			->setCellValue('A12','Other (tamper without alarm)')
			->setCellValue('B12', $res['other_msg'])
			->setCellValue('A13','Totals messages:')
			->setCellValue('B13', $res['total_msg'])
			->setCellValue('A14','Telephone Totals')->mergeCells('A14:B14')
			->setCellValue('A15','Successful telephone calls made')
			->setCellValue('B15', $res['s_phone'])
			->setCellValue('A16','Failed telephone calls made')
			->setCellValue('B16', $res['f_phone'])
			->setCellValue('A17','Totals telephone calls made:')
			->setCellValue('B17', $res['total_phone'])
			->setCellValue('A18','SMS Texts and Email Messages Totals')->mergeCells('A18:B18')
			->setCellValue('A19','Text messages sent')
			->setCellValue('B19', $res['sms_sent'])
			->setCellValue('A20','Emails sent')
			->setCellValue('B20', $res['email_sent']);
		
		$sheet->getStyle('A2')->applyFromArray($style_0);
		$sheet->getStyle('A7:A20')->applyFromArray($style_1);
		$sheet->getStyle('B7:B20')->applyFromArray($style_2);
		
		$sheet->getColumnDimension('A')->setWidth(40);	
		$sheet->getColumnDimension('B')->setWidth(10);	
				
		    	   
		$phpExcelObject->getActiveSheet()->setTitle('Usage summary');
       // Set active sheet index to the first sheet, so Excel opens this as the first sheet
		$phpExcelObject->setActiveSheetIndex(0);		
		
		return $phpExcelObject;
	}				
	
	/*
				array('ownid' 	 => $ownid,
				'date_from'  => date("m/01/Y"),
				'date_to'    => date("m/d/Y"),
				'fixedcount' => $company->getFixedcount(),
				'fixedprice' => $company->getFixedprice(),
				'extraprice' => $company->getExtraprice(),
				'pingfixed'  => $billing['pingfixed'],
				'pingextra'  => $billing['pingextra'],
				'cost_fixed' => $billing['cost_fixed'],
				'cost_extra' => $billing['cost_extra'],
				'cost_total' => $billing['cost_fixed'][''],
				)
	*/
	public function getBilling($params) 
	{
		$stmt = $this->getEntityManager()->getConnection()->prepare('SELECT * FROM getBilling(:ownid, :dt1, :dt2)');
		$stmt->bindValue('ownid', $params['ownid']);		
		$stmt->bindValue('dt1',   $params['dt1']);
		$stmt->bindValue('dt2',   $params['dt2']);
		$stmt->execute();
		$rows = current( $stmt->fetchAll() );	
		
		return $rows;		
	}	
	
	public function exportBilling($phpExcelObject,  $params) 
	{
		$res = $this->getBilling($params);
		
		$phpExcelObject->getProperties()->setCreator("SosMon datafile")
			->setTitle("MS Excel spreadsheet");		
	
		$sheet = $phpExcelObject->setActiveSheetIndex(0);

		$styleArray = array(
			'borders' => array(
				'inside' => array(
					'style' => 'thin',
				),		
				'outline' => array(
					'style' => 'double',
				)
			)
		);
		$style_0 = array(
			'font' => array( 'bold' => true, 'size' => 16),
		);
		
		$style_1 = array(
			'font' => array( 'size' => 12),
		);		
		
		$style_2 = array(
			'font' => array('bold' => true, 'size' => 12),
			'alignment' => array('horizontal' => 1,),
		);
		
		$styleArray0 = array(
			'borders' => array(
				'allborders' => array(
					'style' => 'none',
				)
			)
		);		
		$sheet->getStyle('A1:B6')->applyFromArray($styleArray0);
		
		$sheet->setCellValue('A2', 'BILLING');
		
		$entity = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Customer')->findEntity($params['ownid'], 0);		
		$sheet->setCellValue('A4', 'Company: '.($params['ownid'] == 0 ? 'ALL' : $entity['name']));

		$sheet->setCellValue('A5', 'Period :'. Common::zfdate($params['dt1'], 'sql', 'us') .
								   ' - '.Common::zfdate($params['dt2'], 'sql', 'us') );
		$sheet->mergeCells('A5:B5');

		$sheet->setCellValue('A7','Company settings')->mergeCells('A7:B7')
			->setCellValue('A8','Fixed pings')
			->setCellValue('B8', $res['fixedcount'])
			->setCellValue('A9','Fixed price')
			->setCellValue('B9', $res['fixedprice'])
			->setCellValue('A10','Cost 1 extra ping')
			->setCellValue('B10', $res['extraprice'])
			->setCellValue('A11','Pings summary')->mergeCells('A11:B11')
			->setCellValue('A12','Pings')
			->setCellValue('B12', $res['pingfixed'])
			->setCellValue('A13','Extra pings')
			->setCellValue('B13', $res['pingextra'])
			->setCellValue('A14','Billing')->mergeCells('A18:B18')
			->setCellValue('A15','Fixed')
			->setCellValue('B15', $res['cost_fixed'])
			->setCellValue('A16','Extra')
			->setCellValue('B16', $res['cost_extra'])
			->setCellValue('A17','TOTAL')
			->setCellValue('B17', $res['cost_total']);			
		
		$sheet->getStyle('A2')->applyFromArray($style_0);
		$sheet->getStyle('A7:A17')->applyFromArray($style_1);
		$sheet->getStyle('B7:B17')->applyFromArray($style_2);
		
		$sheet->getColumnDimension('A')->setWidth(40);	
		$sheet->getColumnDimension('B')->setWidth(10);	
				
		    	   
		$phpExcelObject->getActiveSheet()->setTitle('Billing');
       // Set active sheet index to the first sheet, so Excel opens this as the first sheet
		$phpExcelObject->setActiveSheetIndex(0);		
		
		return $phpExcelObject;
	}				
		
	
	public function getCalllog($params) 
	{
		$stmt = $this->getEntityManager()->getConnection()->prepare('SELECT * FROM getCalllog(:ownid, :locationid, :dt1, :dt2)');
		$stmt->bindValue('ownid', $params['ownid']);		
		$stmt->bindValue('locationid', $params['locationid']);	
		$stmt->bindValue('dt1', $params['dt1']);
		$stmt->bindValue('dt2', $params['dt2']);
		$stmt->execute();
		$rows = $stmt->fetchAll() ;	
		
		return $rows;		
	}
	
	public function exportCalllog($phpExcelObject,  $params) 
	{
		$res = $this->getCalllog($params);
		
		$phpExcelObject->getProperties()->setCreator("SosMon datafile")
			->setTitle("MS Excel spreadsheet");		
	
		$sheet = $phpExcelObject->setActiveSheetIndex(0);

		$styleArray = array(
			'borders' => array(
				'inside' => array(
					'style' => 'thin',
				),		
				'outline' => array(
					'style' => 'double',
				)
			)
		);
		$styleArray0 = array(
			'borders' => array(
				'allborders' => array(
					'style' => 'none',
				)
			)
		);		
		$sheet->getStyle('A1:F6')->applyFromArray($styleArray0);
		
		$sheet->setCellValue('A2', 'Call log');
		
		if ($params['ownid'] == -1) {
			$entity = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Alertsystem')->findEntity($params['locationid'], 0);
			$sheet->setCellValue('A4', 'Location: '.($params['locationid'] == 0 ? 'ALL' : $entity['name']));
		}
		else {
			$entity = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Customer')->findEntity($params['ownid'], 0);		
			$sheet->setCellValue('A4', 'Company: '.($params['ownid'] == 0 ? 'ALL' : $entity['name']));
		}
		$sheet->setCellValue('A5', 'Period :'. Common::zfdate($params['dt1'], 'sql', 'us') .
								   ' - '.Common::zfdate($params['dt2'], 'sql', 'us') );
		$sheet->mergeCells('A5:C5');
		
		$list[] = array('id'  	  => 1,
					  'dt'  	  => '10/01/15 11:18:23',
					  'customer'  => 'Some customer', 
					  'esn' 	  => '0-12345',
					  'location'  => 'Ecke',
					  'dtcall'    => '10/01/15 11:26:23',
					  'delta'     => 8,
		);
		
		
		$sheet->setCellValue('A7','Company')
			->setCellValue('B7','Alarm')
			->setCellValue('C7','ESN')
			->setCellValue('D7','Location')
			->setCellValue('E7','Call')
			->setCellValue('F7','Delay');
		$sheet->getStyle('A7:F7')->applyFromArray($styleArray);	
		$nr = 7;
		
		foreach($res as $row) {
			$nr++;
			
			$sheet->setCellValue('A'.$nr, $row['customer'])
				->setCellValue('B'.$nr,$row['dt'])
				->setCellValue('C'.$nr,$row['esn'])
				->setCellValue('D'.$nr,$row['location'])
				->setCellValue('E'.$nr,$row['dtcall'])
				->setCellValue('F'.$nr,$row['delta']);
	   
		}
		$sheet->getStyle('A8:F'.$nr)->applyFromArray($styleArray);	
		
		$sheet->getColumnDimension('A')->setWidth(30);	
		$sheet->getColumnDimension('B')->setWidth(10);	
		$sheet->getColumnDimension('C')->setWidth(10);	
		$sheet->getColumnDimension('D')->setWidth(30);	
		$sheet->getColumnDimension('E')->setWidth(10);	
		$sheet->getColumnDimension('F')->setWidth(10);			
		
		$phpExcelObject->getActiveSheet()->setTitle('Call log');
       // Set active sheet index to the first sheet, so Excel opens this as the first sheet
		$phpExcelObject->setActiveSheetIndex(0);		
		
		return $phpExcelObject;
	}				
	
		
	
	
}