<?php
namespace Acme\SosMonBundle\Entity;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\NoResultException;
use \DateTime;
use Acme\SosMonBundle\Model\Common;

class RawRepository extends EntityRepository 
{
	/**
	* Get all Raws
	*
	*@param int $rows
	*@param int $offset
	*@param string $dt1
	*@param string $dt2
	*@param int $success
	*@return $res array
	*/
	public function getList($rows, $offset, $dt1, $dt2, $success)
	{	

		$stmt = $this->getEntityManager()->getConnection()->prepare('SELECT * FROM getRaws( :dt1, :dt2, :success, :rows, :offset)');
		$stmt->bindValue('dt1', $dt1);
		$stmt->bindValue('dt2', $dt2);
		$stmt->bindValue('success', $success);		
		$stmt->bindValue('rows', $rows);
		$stmt->bindValue('offset', $offset);
		$stmt->execute();
		$rows = $stmt->fetchAll();	
				 
		$res = array();
		foreach ($rows as $row) {
			$res[] = array(
				'id' 		=> (int)$row['id'],					
				'dt' 		=> $row['dt'],				
				'success'	=> (int)$row['success'],
				'message' 	=> $row['message'],	
			);
		}
		return array("total" => count($res), 
					 "rows"  => $res);
	}	
	
	/**
	* Get entity Raw record
	*
	* @param int $id
	* @param int $ownid	
	* @return entity $entity
	*/			
	public function findEntity($id)
	{
		$entity = $this->find($id);	
		if (!$entity) {
			$entity = new Raw(); 
			$entity->setDt( new DateTime);
		}
				
		return array(			
			'id' 		=> (int)$entity->getId(),		
			'dt' 		=> Common::fmtDateTime( $entity->getDt() ),	
			'success'	=> (int)$entity->getSuccess(),
			'message' 	=> $entity->getMessage(),
			'messageid' => $entity->getMessageid(),
		);					
	}
	
	/**
	* Save Raws
	*
	* @param array $var  

	* @return entity id
	*/
	public function saveEntity( $var )
	{
		$id = 0;
		if ( isset($var['message']) ) {
			$entity = new Raw();	
			$entity->setMessage(  (string) $var['message'] )
				   ->setSuccess( isset($var['success']) ? (int) $var['success'] : 0 )
				   ->setDt( new DateTime)
				   ->setMessageid( isset($var['messageid']) ? $var['messageid'] : '');
					
			$em = $this->getEntityManager();				
			$em->persist($entity);
			$em->flush();
			$id = $entity->getId();	
		}
		
		return $id;			
	}
	
	/**
	* Set finalize
	*
	* @param int $id
	* @param int $success
	* @return @id
	**/
	public function setFinalize($id, $success)
	{	
		$entity = $this->find( $id );	
		if ($entity) {
			$entity->setSuccess( (int) $success );
			$em = $this->getEntityManager();				
			$em->persist($entity);
			$em->flush();			
		}		
		return $id;
	}	
	
	/*	
	public function parseMessage( $message )
	{
		return json_decode(json_encode(simplexml_load_string( $message )), TRUE);
	}


	* Set finalize
	*
	* @param int $id
	* @param int $success
	* @return 1

	public function rawPackage( $message )
	{	
		$id = $this->saveEntity( array(  'message'	=> $message, 
										 'success' => 0,
									   ));	
		
		// parsing
		$amsg = $this->parseMessage( $message );	
		$success = 1;	

		$msg = $amsg['stuMessage'];
		
		if ( isset( $msg['esn'] ) ) { // case of 1 message at package
			$eventid = $this->proceedMessage( $id, $msg );
			$success = ($eventid == 0) ? 0 : 1;
		}
		else {
			foreach ($msg as $m) { // case of more then 1 message at package
				$eventid = $this->proceedMessage( $id, $m );
				if ($eventid == 0) {
					$success = 0; // At least one fail means all fail
				}			
			}
		}
	
		$this->setFinalize($id, $success);	 
		return $id;
	}	
		**/
	/**
	* Set finalize
	*
	* @param int $id
	* @param int $success
	* @return 1

	public function proceedMessage( $id, $msg )
	{
		$date = new DateTime();
		$em = $this->getEntityManager();
		$event   = $em->getRepository('AcmeSosMonBundle:Event')->findEntity(0, 0);
		$station = $em->getRepository('AcmeSosMonBundle:Station')->findOneByEsn( $msg['esn'] );
		$eventid = 0;					
		
		if ($station) {
			$event['rawid'] 		= $id;		
			$event['ownid'] 		= $station->getOwnid();
			$event['stationid'] 	= $station->getId();
			$event['status']		= $station->getStatus();
			$event['ibuttonid']		= -1;
			$event['keyholderid']	= -1;
			$event['response']	    = 0;
		//	if ( $msg['payload'] == 'N' ) {
				$event['latitude'] 		= $station->getLatitude();
				$event['longitude']		= $station->getLongitude();
		//	}
			$event['charge'] 		= ($station->getCharge() < 3) ? 100 : $station->getCharge() - 1;
			$event['edata'] 		= $msg['payload'];
			$event['dt']            = Common::fmtDateTime( $date->setTimestamp( (int)$msg['unixTime'] ) );
			
			// here payload must be proceeded to get 1) status  2)ibutton	
			$r = rand ( 0 , 9 );
			$event['status'] = ($r == 3) ? 3: 1;
			// end of payload proceed
			

			if ( ($event['ownid'] > 0) && ($event['stationid'] > 0) && ($event['latitude'] <> 0) && ($event['longitude'] <> 0) && ($event['charge'] > 0) && ($event['status'] > 0) ) {	
				// create new event
				$eventid = $em->getRepository('AcmeSosMonBundle:Event')->saveEntity( $event ); 
				$em->getRepository('AcmeSosMonBundle:Event')->saveResponse($eventid, 0);
			
				// upgrade station status
				$station->setLatitude ( $event['latitude'] );
				$station->setLongitude( $event['longitude'] );
				$station->setStatus( (int)$event['status'] );
				$station->setCharge((int)$event['charge'] );
				$station->setEventid( $eventid );
				$em->getRepository('AcmeSosMonBundle:Station')->commitEntity( $station );
				
				
			}
		}
		
		return $eventid;		
	}	
		**/	
	
}

