<?php
namespace Acme\SosMonBundle\Entity;

use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\NoResultException;
use \DateTime;
use Acme\SosMonBundle\Model\Common;

class StationRepository extends EntityRepository 
{

	/**
	* Get all Stations
	*
	* @return $res array
	*/
	public function getTree($ownid)
	{	
		$stmt = $this->getEntityManager()->getConnection()->prepare('SELECT * FROM getStations_tree(:ownid) order by alertname, alertsystemid, esn' );
		$stmt->bindValue('ownid', $ownid);
		$stmt->execute();
		$rows = $stmt->fetchAll();	
				 
		$res = array();
		$aStat =  Common::getUnitStatuses();		
        $today = new DateTime;
		$items = 0;
		$og = -1;
		$id = 0;
		foreach ($rows as $row){	
			$id++;
				if ($og <> $row['alertsystemid']) {
				
					if ($items > 0) {
						$res[] = $obj;
					}
					
					$obj = array('id'  	=> $id,
								'entityid' 	  => (int)$row['alertsystemid'],
								'displayname' => $row['alertname'],
								'charge'	  => -1,
								'level'		  => 1,
								'children'    => array()
						);	
					$id++;
					
					$og = $row['alertsystemid'];
					$items = 1;
				}
				if ((int)$row['id'] > 0) { // -1 - empty system
					$items ++;
				
					$obj['children'][] = array( 
						'id' 		=> $id,	
						'entityid' 	=> (int)$row['id'],	
						'level'		=> 2,	
						'displayname'	=> $row['esn'],	
						'sname' 	=> $row['name'],						
						'name' 		=> $row['name'],	
						'location' 	=> $row['location'],				
						'note' 		=> $row['note'],		
						'esn' 		=> $row['esn'],
						'type'		=> (int)$row['type'],	
						'dstart' 	=> Common::zfdate($row['dstart'], 'mysql', 'us'),	
						'status'	=> (int)$row['status'],
						'info'      => $aStat[(int)$row['status']].(((int)$row['status']  == 4) ?'; No comm in '.$row['event_days'].' days' :''),
						'longitude' => (float)$row['longitude'],				
						'latitude' 	=> (float)$row['latitude'],	
						'dms'		=> Common::DecimalToDMS((float)$row['latitude'], false).' '.Common::DecimalToDMS((float)$row['longitude'], true),						
						'charge'	=> (int)$row['charge'],
						'pricing'	=> $row['pricing'],	
						'dcharge' 	=> Common::zfdate($row['dcharge'], 'mysql', 'us'),	
						'devent'    => ((int)$row['event_status'] == 0) ? '< None>': $row['event_dt'].' '.$aStat[(int)$row['event_status']].' '.$row['event_ibutton'].' '.$row['event_keyholder'],			
					);	
				}				
		}
		if ($items > 0) {
			$res[] = $obj;
		}
		
		return $res;
	}
	

	/**
	* Get all Stations
	* @param userid
	* @param $opt (1/2)
	* @param $alertsystemid
	* @return $res array
	*/
	public function getList($userid, $opt, $alertsystemid = 0)
	{	
		$stmt = $this->getEntityManager()->getConnection()->prepare('SELECT * FROM getStations(:userid, :alertsystemid) order by esn');
		$stmt->bindValue('userid', $userid);
		$stmt->bindValue('alertsystemid', $alertsystemid);
		$stmt->execute();
		$rows = $stmt->fetchAll();	
		
//		$this->get('call_sqllog')->regSqllog($userid, 
//											 'SELECT * FROM getStations(:userid, :alertsystemid) order by esn', 
//											 json_encode(array($userid, $alertsystemid)));							  
/*		
		$this->getEntityManager()->getRepository('AcmeSosMonBundle:Sqllog')
			->saveEntity(array( 'userid' => $userid,
								'query'  => 'SELECT * FROM getStations(:userid, :alertsystemid) order by esn',
								'params' => json_encode(array($userid, $alertsystemid)),
							  ));		
*/				 
		$res = array();
		$aStat =  Common::getUnitStatuses();
		
		if ($opt == 1) {
			$res[] = array(
				'id' 		=> 0,			
				'name' 		=> '< ALL >',	
			);		
		}
		
		if ($opt == 2) {
			$res[] = array(
				'id' 		=> 0,			
				'name' 		=> '< NONE >',	
			);		
		}		
		
		foreach ($rows as $row) {
			$res[] = array(
				'id' 		=> (int)$row['id'],	
				'entityid' 	=> (int)$row['id'],	
				'level'		=> 2,
				'displayname' => $row['esn'],	
				'sname' 	=> $row['name'],				
				'name' 		=> $row['name'].' ('.$row['esn'].')',	
				'location' 	=> $row['location'],				
				'note' 		=> $row['note'],		
				'esn' 		=> $row['esn'],
				'type'		=> (int)$row['type'],	
				'dstart' 	=> Common::zfdate($row['dstart'], 'mysql', 'us'),	
				'status'	=> (int)$row['status'],
				'info'      => $aStat[(int)$row['status']].(((int)$row['status']  == 4) ?'; No comm in '.$row['event_days'].' days' :''),
				'longitude' => (float)$row['longitude'],				
				'latitude' 	=> (float)$row['latitude'],	
				'dms'		=> Common::DecimalToDMS((float)$row['latitude'], false).' '.Common::DecimalToDMS((float)$row['longitude'], true),
				'charge'	=> (int)$row['charge'],
				'pricing'	=> (int)$row['pricing'],
				'dcharge' 	=> Common::zfdate($row['dcharge'], 'mysql', 'us'),	
				'devent'    => ((int)$row['event_status'] == 0) ? '< None>': $row['event_dt'].' '.$aStat[(int)$row['event_status']].' '.$row['event_ibutton'].' '.$row['event_keyholder'],			
			);
		}
		return $res;
	}	
	
	/**
	* Get entity SosMon station
	*
	* @param int $id
	* @param int $ownid	
	* @return entity $entity
	*/			
	public function findEntity($id, $company )
	{
		$entity = $this->find($id);	
		if (!$entity) {
			$entity = new Station(); 
			$entity->setOwnid( $company->getOwnid() );
			$entity->setDstart( new DateTime );
			$entity->setLatitude( $company->getDeflat() );
			$entity->setLongitude( $company->getDeflong() );
			$entity->setStatus( 2 ); // OFF
		}
		
		$aStat =  Common::getUnitStatuses();
				
		return array(			
				'id' 		=> (int)$entity->getId(),		
				'ownid' 	=> (int)$entity->getOwnid(),	
				'name' 		=> $entity->getName(),
				'location' 	=> $entity->getLocation(),		
				'note' 		=> $entity->getNote(),	
				'esn' 		=> $entity->getEsn(),
				'type'		=> (int)$entity->getType(),
				'dstart' 	=> Common::fmtDateTime($entity->getDstart()),
				'status'	=> (int)$entity->getStatus(),
				'longitude' => (float)$entity->getLongitude(),		
				'latitude' 	=> (float)$entity->getLatitude(),
				'charge'	=> (int)$entity->getCharge(),
				'pricing'	=> (int)$entity->getPricing(),
				'dcharge' 	=> Common::fmtDateTime($entity->getDcharge()),
				'eventid' 	=> (int)$entity->getEventid(),
				'info'      => $aStat[(int)$entity->getStatus()],	
				'devent'    => '',					
					);					
	}
	
	/**
	* Save Station as entity
	*
	* @param entity $entity  
	*/
	public function commitEntity( $entity )
	{
		$em = $this->getEntityManager();				
		$em->persist($entity);
		$em->flush();
	}
	
	/**
	* Save Station
	*
	* @param array $var  
	* @return entity id
	*/
	public function saveEntity( $var )
	{
		$entity = $this->find((int)$var['id']);	
		if (!$entity) {
			$entity = new Station();
		}
			
		$entity->setName( $var['name'] );
		$entity->setOwnid( (int)$var['ownid'] );
		$entity->setLocation( $var['location'] );		
		$entity->setNote( $var['note'] );	
		$entity->setEsn( $var['esn'] );
		$entity->setType( (int)$var['type'] );
		$entity->setDstart( Common::zfdate($var['dstart'], 'us', 'date') );
		$entity->setStatus( (int)$var['status'] );
		$entity->setLongitude( $var['longitude'] );
		$entity->setLatitude( $var['latitude'] );
		$entity->setCharge( (int)$var['charge'] );
		$entity->setPricing( (int)$var['pricing'] );		
	//	$entity->setDcharge( Common::zfdate($var['dcharge'], 'us', 'date')  );
		$entity->setEventid( (int) $var['eventid'] );	
					
		$em = $this->getEntityManager();				
		$em->persist($entity);
		$em->flush();

		$id = $entity->getId();  	

		return $id;			
	}	
	
	/**
	* checkEntity
	*
	* @param array $var
	* @return Bool
	**/
	public function checkEntity( $var )
	{
		$res = array('success' => true,
					 'message' => '' );
					 
		$stations = $this->findBy(array('esn' => $var['esn'], 'deleted' => 0));
		foreach( $stations as $item) {
			if ($item->getId() != (int)$var['id']) {
				$res['success'] = false;
				$res['message'] = 'Esn '.$var['esn'].' already exists';
			}
		}
		
		return $res;
	}	
	
	/**
	* Delete customer
	*
	* @param int $id
	* @return 1
	**/
	public function deleteEntity( $id )
	{	
		$entity = $this->find( $id );	
		if ($entity) {
			$em = $this->getEntityManager();						
			if ( $this->canDelete( $entity )) {
				$em->remove($entity);
			}
			else {
				$entity->setDeleted( 1 ); 
				$em->persist($entity);				
			}
			$em->flush();		
	    }		 								 
		return $entity;
	}	
	
	/**
	* Check if entity can be deleted or not
	*
	* @param $ntity
	* @return bool
	**/
	public function canDelete( $entity )
	{	
		$flag = false;
		return $flag;
	}
	
	/**
	* Predefined array of unit types
	*
	* @return array access types
	**/
	public function getUnitTypes()
	{	
		return array(  	0 => 'Regular',
					);
	}	
	
	/**
	* Predefined array of unit pricing
	*
	* @return array 
	**/
	public function getUnitPricing()
	{	
		return array(  	0 => 'Regular',
						1 => 'Dyno',
					);
	}	
	
	/**
	* Array of markers for each registered stations
	*
	* @return array access types
	**/	
	public function getMarkersForStations($userid, $xcoord, $ycoord)
	{
		$x_max = $xcoord;
		$y_max = $ycoord;
		$x_min = $xcoord;
		$y_min = $ycoord;
		
		$markers = array('x_center' => 0,
						 'y_center' => 0,
						 'zoom' 	=> 10,
						 'items' 	=> array());
						 
		$stmt = $this->getEntityManager()->getConnection()->prepare('SELECT * FROM getStations( :userid, 0 )');
		$stmt->bindValue('userid', $userid);
		$stmt->execute();
		$rows = $stmt->fetchAll();	
		
		if (count($rows) > 0) {
			$x_max = -180;
			$y_max = -180;
			$x_min = 180;
			$y_min = 180;		

			// which status => which marker
			$aIcon = Common::getMarkerType();
				
			foreach ($rows as $row) {
		
				$y_max = ($y_max < (float)$row['latitude']) ? (float)$row['latitude']: $y_max;
				$y_min = ($y_min > (float)$row['latitude']) ? (float)$row['latitude']: $y_min;

				$x_max = ($x_max < (float)$row['longitude']) ? (float)$row['longitude']: $x_max;
				$x_min = ($x_min > (float)$row['longitude']) ? (float)$row['longitude']: $x_min;
					
				$markers['items'][] = array('x_c'   => (float)$row['longitude'],
											'y_c'   => (float)$row['latitude'],
											'text'  => $row['name'],
											'id'    => (int)$row['id'],
											'entity_kind'    => 1,
											'state' => $aIcon[$row['status']],
											);
			}
		}
		
		$markers['x_center'] = ($x_max+$x_min)/2;
		$markers['y_center'] = ($y_max+$y_min)/2;
		$markers['zoom']     =  Common::detectZoom(MAX( ABS($x_max - $x_min), ABS($y_max - $y_min)));
		
		return $markers;		
	}
	
	/**
	* Array of stations statuses
	*
	* @return array 
	**/	
	public function getCurrentStatus($userid)
	{					 
		$stmt = $this->getEntityManager()->getConnection()->prepare('SELECT * FROM getCurrentStatus( :userid )');
		$stmt->bindValue('userid', $userid);
		$stmt->execute();
		$rows = current($stmt->fetchAll());
		
		return $status = array('alerts' => $rows['alert'],
							   'warns'  => $rows['low'],
							   'missing'=> $rows['noresp'],
							   'time'   => date("m/d/Y h:i:s")." ( Next update: ".$rows['rrate']." m. )"
						);	
	}
	
	/**
	* Import data
	* @param $row array
	* @return array 
	**/	
	public function importData($ownid, $row)
	{					 				
		$stmt = $this->getEntityManager()->getConnection()->prepare('SELECT * FROM importStation( :ownid, :alert, :station, :esn, :location, :note, :latitude, :longitude )');
		$stmt->bindValue('ownid',   $ownid);
		$stmt->bindValue('alert',   $row['Alert']);
		$stmt->bindValue('station', $row['Station']);
		$stmt->bindValue('esn',     $row['ESN']);
		$stmt->bindValue('location',$row['Location']);
		$stmt->bindValue('note',    $row['Note']);
		$stmt->bindValue('note',    $row['Note']);
		$stmt->bindValue('note',    $row['Note']);
		$stmt->bindValue('latitude',    $row['latitude']);
		$stmt->bindValue('longitude',    $row['longitude']);
		$stmt->execute();	
		return current($stmt->fetchAll());	
	}	
	
	
}