<?php
namespace Acme\SosMonBundle\Entity;

use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Component\Security\Core\User\UserProviderInterface;
use Symfony\Component\Security\Core\Exception\UsernameNotFoundException;
use Symfony\Component\Security\Core\Exception\UnsupportedUserException;
use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\NoResultException;
use Acme\SosMonBundle\Model\Common;

class UserRepository extends EntityRepository implements UserProviderInterface
{
 	/**
	* UserProviderInterface methods
	*/
    public function loadUserByUsername($username)
    {
		$user = $this->findOneBy(
			array('username' => $username, 'deleted' => 0)
		);	
		
		if (is_object($user) && ($user->getOwnid() > 0) ) {
			$company = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Customer')->find($user->getOwnid()); 		
			if ($company->getActive() == 0) {
				$user = null;
				throw new UnsupportedUserException( 'User is not found' );
			}
		}
		
        return $user;
    }
	/**
	* UserProviderInterface methods
	*/
    public function refreshUser(UserInterface $user)
    {
        $class = get_class($user);
        if (!$this->supportsClass($class)) {
            throw new UnsupportedUserException(
                sprintf(
                    'Instances of "%s" are not supported.',
                    $class
                )
            );
        }

        return $this->find($user->getId());
    }
	/**
	* UserProviderInterface methods
	*/
    public function supportsClass($class)
    {
        return $this->getEntityName() === $class
            || is_subclass_of($class, $this->getEntityName());
    }
	

	/**
	* Get all SosMon users for specified company
	*
	* @param int $ownid 
	* @param int $opt 
	* @return $res array
	*/
	public function getList( $ownid, $opt )
	{
		$users = $this->findBy(array('ownid' => $ownid, 'deleted' => 0), array('username' => 'ASC'));		
		$access = Common::getAccessTypes();	
			 
		$res = array();
		if ($opt == 1) {
			$res[] = array(
				'userid' 	=> 0,			
				'name' 		=> 'ALL',	
			);		
		}		
		
		foreach ($users as $user) {
			$res[] = array(
				'userid' 	    => $user->getId(),
				'ownid' 	    => $user->getOwnid(),
				'acclevel' 	    => $user->getAcclevel(),
				'accname'       => $access[ $user->getAcclevel() ],
				'login' 	    => $user->getUsername(),
				'name' 		    => $user->getName(),
				'phone' 	    => $user->getPhone(),
				'mobile'	    => $user->getMobile(),
				'email'		    => $user->getEmail(),
				'accword'	    => $user->getAccword(),
				'active'	    => $user->getActive(),
				'alertcontactid'=> $user->getAlertcontactid(),
				'chgpassword'   => $user->getChgpassword(),
				'alertsystems'  => $this->getEntityManager()->getRepository('AcmeSosMonBundle:Alertusers')->getDescriptionForUser($user->getId()),
			);
		}
		return $res;
	}	
	
	/**
	* Get all SosMon users for specified company
	*
	* @param int $ownid 
	* @return $res array
	*/
	public function getUsersForCompany( $ownid )
	{	
		$res = array();
		$res[] = array(
			'userid' 	=> 0,			
			'name' 		=> 'ALL',	
		);
		
		if ($ownid > 0) {
			$users = $this->findByOwnid($ownid, array('username' => 'ASC'));
		
			foreach($users as $usr) {
				$res[] = array(
					'userid' 	=> $usr->getId(),			
					'name' 		=> $usr->getUsername(),	
				);
			}
		}
		return $res;
	}	

	/**
	* Get entity SosMon location
	*
	* @param int $id
	* @return entity $entity
	*/			
	public function findEntity($id)
	{
		$entity = $this->find($id);	
		if (!$entity) {
			$entity = new User(); 
			$entity->setAcclevel(2);
			$entity->setAccword('0222110');
		}
		
		return array(	'userid' 	=> (int)$entity->getId(),			
						'name' 		=> $entity->getName(),
						'login' 	=> $entity->getUsername(),
						'password' 	=> $entity->getPassword(),		
						'phone' 	=> $entity->getPhone(),						
						'mobile' 	=> $entity->getMobile(),
						'email' 	=> $entity->getEmail(),
						'accword'   => $entity->getAccword(),
						'acclevel' 	=> (int)$entity->getAcclevel(),
						'ownid' 	    => (int)$entity->getOwnid(),
						'active'	    => (int)$entity->getActive(),
						'alertcontactid'=> (int)$entity->getAlertcontactid(),
						'chgpassword'   => (int)$entity->getChgpassword(),
					);		
	}
	
	/**
	* Get systems for  User
	*
	* @param $id int 
	* @return $res array
	*/
	public function getSystems($id, $ownid)
	{	
		$list = $this->getEntityManager()->getRepository('AcmeSosMonBundle:Alertsystem')->getList($ownid, 0);
	
		$res = array();
		foreach($list as $row) {
			$res[] = array( 'id'   => $row['id'],
							'name' => $row['name'],
							'chk'  => $this->getEntityManager()->getRepository('AcmeSosMonBundle:Alertusers')->isExists($row['id'], $id),
						);		
		}		
		return $res;
	}
	
	/**
	* Get access for  User as array
	*
	* @param $id int 
	* @return $ref array
	**/
	public function getAccessAsArray( $accstr )
	{	
		$ref = Common::getAccessArray( 0 );
		foreach( $ref as $key => $value) {
			$ref[ $key ] = $this->getSectionAccess( $accstr, $key );
		}
		return $ref;
	}
	
	/**
	* Get access for  User
	*
	* @param $id int 
	* @return $res array
	*/
	public function getAccess( $accstr )
	{	
		$res = array();
		
		$row = Common::getAccessArray();
		$row['id'] = 0;
		$row['name'] = 'None';
		$res[] = $row;
		$row = Common::getAccessArray();
		$row['id'] = 1;
		$row['name'] = 'Read';
		$res[] = $row;
		$row = Common::getAccessArray();
		$row['id'] = 2;
		$row['name'] = 'Full';
		$res[] = $row;		

		$res[ (int)$this->getSectionAccess( $accstr, 'users' ) ]['users' ]          = 1;
		$res[ (int)$this->getSectionAccess( $accstr, 'contacts' ) ]['contacts']     = 1;
		$res[ (int)$this->getSectionAccess( $accstr, 'keyholders' ) ]['keyholders'] = 1;
		$res[ (int)$this->getSectionAccess( $accstr, 'ibuttons' ) ]['ibuttons']     = 1;
		$res[ (int)$this->getSectionAccess( $accstr, 'stations' ) ]['stations']     = 1;
		$res[ (int)$this->getSectionAccess( $accstr, 'events' ) ]['events']         = 1;
		$res[ (int)$this->getSectionAccess( $accstr, 'admin' ) ]['admin']           = 1;
		$res[ (int)$this->getSectionAccess( $accstr, 'call' ) ]['call']             = 1;
		$res[ (int)$this->getSectionAccess( $accstr, 'loghistory' ) ]['loghistory'] = 1;
		$res[ (int)$this->getSectionAccess( $accstr, 'logsummary' ) ]['logsummary'] = 1;
		$res[ (int)$this->getSectionAccess( $accstr, 'logcalllog' ) ]['logcalllog'] = 1;
		
		
		$ret = array();
		$ret[] = $res[2];
		$ret[] = $res[1];
		$ret[] = $res[0];
		
		return $ret;
	}
	
	/**
	* generateRandomString
	* @param int $length
	* @return string
	**/
	function generateRandomString($length = 6) 
	{
		$characters = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ';
		$randomString = '';
		for ($i = 0; $i < $length; $i++) {
			$randomString .= $characters[rand(0, strlen($characters) - 1)];
		}
		return $randomString;
	}
	
	
	
	/**
	* create temporary admin
	* @param int $ownid
	* @return array
	**/
	public function createAdmin($ownid, $prefix, $encoder)
	{
		// if admin is already exists then create adminNN
		$nr = 0;
		while (true) {
			$login = 'admin'.($nr == 0 ? '' : $nr);
			if ( $this->checkEntity(0, $prefix.'_'.$login) ) {
				break;
			}
			else {
				$nr++;
			}
		}					
		
		$admin = array( 'userid'      => 0,
						'name'        => 'Temporary administrator account',
						'prefix'      => $prefix.'_',
						'login'       => $login,
						'password'    => $this->generateRandomString(),
						'phone'       => '',
						'mobile'      => '',
						'email'       => '',
						'active'      => 1,
						'accword'     => str_repeat('2', 20),
						'ownid'       => $ownid,
						'acclevel'    => 1,
						'alertcontactid' => 0,
						'asystems'    => '',
						'chgpassword' => 1,
				);
		$id = $this->saveEntity( $admin, $encoder );
		$admin['userid'] = $id;
		return $admin;
	}
	/**
	* Save User
	*
	* @param array $var  
	* @return entity id
	*/
	public function saveEntity( $var, $encoder )
	{
		$entity = $this->find((int)$var['userid']);	
		if (!$entity) {
			$entity = new User();
		}
	
		$entity->setName(     $var['name'] ); 
		$entity->setUsername( ((int)$var['ownid'] > 0) ? $var['prefix'].$var['login'] : $var['login']);
		$entity->setOwnid(    (int)$var['ownid']);		
		$entity->setActive(   (int)$var['active']);			
		$entity->setAcclevel( (int)$var['acclevel'] ); // acclevel is obsolet. Access array is ON now 
		$entity->setAccword(  $var['accword']);
		$entity->setPhone(    $var['phone']);
		$entity->setMobile(   $var['mobile']);
		$entity->setEmail(    $var['email']);	
		$entity->setAlertcontactid( (int)$var['alertcontactid']);
		$entity->setDeleted(  0 );
		if ( isset($var['chgpassword']) ) {
			$entity->setChgpassword(    (int) $var['chgpassword']);	
		}
			

		//$encoded = $encoder->encodePassword($entity, $var['password']);	
		$encoded = $var['password'];		
		$entity->setPassword( $encoded );		
				
		$em = $this->getEntityManager();				
		$em->persist($entity);
		$em->flush();
		
		$id = $entity->getId();
		
		return $id;			
	}
	
	/**
	* Save User Password. Is not using by now
	*
	* @param int $id
	* @param array $var  
	* @return entity id
	*/
	public function savePassword( $id, $password)
	{
			
	    $entity = $this->find($id);
		
		if ( $entity ) {
			$entity->setPassword( $password );
			$entity->setChgpassword( 0 );
			
			$em = $this->getEntityManager();				
			$em->persist($entity);
			$em->flush();			
		}
					 
		return $id;
		
	}	
	
	/**
	* Check if user is available
	* @param int $id
	* @param string $login
	*
	* @return Boolean)
	**/
	public function checkEntity($id, $login)
	{
		$user = $this->findOneByUsername( $login );
		$found = false;
		if ($user) {
			$found = ($user->getId() != $id);
		}
		
		return  !$found;
	}
			
	/**
	* Users set active status
	*
	* @param int $id
	* @return entity id
	*/
	public function activeEntity( $id)
	{	
	    $entity = $this->find($id);
		
		if ( $entity ) {
			$active = (($entity->getActive() == 0 ) ? 1 : 0);
			$entity->setActive( $active );
			
			$em = $this->getEntityManager();				
			$em->persist($entity);
			$em->flush();			
		}					 
		return $id;
	}	
	
	/**
	* Users set password change on/off
	*
	* @param int $id
	* @param int $chgpassword
	* @return entity id
	*/
	public function chgPasswordEntity( $id, $chgpassword)
	{	
	    $entity = $this->find($id);
		
		if ( $entity ) {
			$entity->setChgpassword( $chgpassword );
			
			$em = $this->getEntityManager();				
			$em->persist($entity);
			$em->flush();			
		}					 
		return $id;
	}	
		
	/**
	* Delete user
	*
	* @param int $id
	* @return $entity
	**/
	public function deleteEntity( $id )
	{	
		$entity = $this->find( $id );	
		if ($entity) {
			$em = $this->getEntityManager();						

			$entity->setDeleted( 1 ); 
			$em->persist( $entity );				
			$em->flush();		
	    }		 								 
		return $entity;
	}	
	
	
	/**
	* @param string $word 
	* @param string $section
	*
	* @return int access level(0,1,2)
	**/
	public function getSectionAccess( $word, $section )
	{	
		$ref = Common::getAccessArray( 1 );
		
		$val = 0;		
		if (array_key_exists( $section, $ref )) {		
			$val = (int)substr($word.str_repeat('0',40), $ref[ $section ], 1);
		}
		return $val;
	}	
	
	
}