<?php
namespace Acme\SosMonBundle\Model;  
use Acme\SosMonBundle\Event\PackageEvent; 
use Acme\SosMonBundle\Event\CampaignEvent; 
use Acme\SosMonBundle\Model\CampaignStatus;
 
class AlertcallProcessing
{
	private $em;
	private $call;
	private $logger;
	private $campaign;
	
	public function __construct($em, $call, $logger, $campaign) {
		$this->em       = $em;
		$this->call     = $call;
		$this->logger   = $logger;
		$this->campaign = $campaign;
	} 

 /*
    onStartCampaign Event Processing
	
 */
    public function onStartCampaignEvent(CampaignEvent $event) 
	{		
	//	$this->logger->warn( 'start '.$event->getEventid().'-'.$event->getStationid() );
		$this->campaign->removeCampaign( $event->getEventid() );
		
		$cnt = $this->campaign->createCampaign( $event->getEventid(), $event->getStationid(), $this->SpellEvent( $event->getEventid() ) );
		if ($cnt > 0) {
			$result = $this->campaign->getFirstForCampaign( $event->getEventid() );
			
		//	$this->logger->warn( 'Start campaign : '.json_encode( $result ) );
			if ( $result->getFound() ) {
				$this->call->sendCall( $result->getPhone(), $result->getMessage() );	
				$this->call->regPhoneNotification($result->getContactid(), $result->getEventid(), 0);
			}
			else {
			// Exception. Error next phone was not found
			}
		}
		else {
		// Exception. Location has no contact
		}			   
	}	
 /*
    OnAlertcall Event Processing. Call summary processing
	
 */
    public function onAlertcallEvent(PackageEvent $event) 
	{		
	//	$this->logger->warn( 'call summary : '.$event->getMessage() );

		try {
			$amsg = Common::object_to_array( json_decode( $event->getMessage())  );		
		}
		catch(Exception $e) {
			// Exception. Package can not be decoded
			// Just ignore it
			return;
		}
						
		$phone   = isset( $amsg['@attributes']['phonenumber'] ) ? $amsg['@attributes']['phonenumber'] : '';
		$status  = isset( $amsg['@attributes']['status'] )      ? $amsg['@attributes']['status'] : '';
		$comment = isset( $amsg['@attributes']['comment'] )     ? $amsg['@attributes']['comment'] : '' ;
		$keypress= isset( $amsg['prompts']['prompt']['@attributes']['keypress'] )? $amsg['prompts']['prompt']['@attributes']['keypress'] : '' ;	

		//! leading "1" is have to be removed
		if ((strlen($phone) > 0) && (substr($phone,0, 1) == '1')) {
			$phone = substr($phone, 1, strlen($phone));
		}

		
		if (($status == VoiceshotConfig::successStatus() )&&($keypress == VoiceshotConfig::keyConfirm())) {
			$result = $this->campaign->stopCampaign( $phone );
			$this->call->regPhoneNotification($result->getContactid(), $result->getEventid(), 1);			
//			$this->logger->warn( 'success. stop campaign : '.$phone );
		}
		else {
//			            $this->logger->warn( 'fail call : '.$phone );
			$result = $this->campaign->getNextForCampaign( $phone, $this->logger );
//			            $this->logger->warn( 'Campaign status : '.json_encode( (array) $result ) );
			if ( $result->getFound() ) {
				$this->call->sendCall( $result->getPhone(), $result->getMessage() );
				$this->call->regPhoneNotification($result->getContactid(), $result->getEventid(), 0);				
//				$this->logger->warn( 'next call : '.$result->getPhone() );
			}
		}	
	}
	
	/*
	**  Event message. Spell difference between current event and previous event for station
	**  @param int $eventid
	**  @return string
	*/
	public function SpellEvent( $eventid )
	{
		$event = $this->em->getRepository('AcmeSosMonBundle:Event')->findEntity( $eventid, 0 );	
		
		$payload1 = Payload::Decode9byte( $event['edata'] );
		$payload2 = Payload::Decode9byte( $this->em->getRepository('AcmeSosMonBundle:Event')->findPreviousEdata( $eventid, $event['stationid'] ) );
		
		$message = '';
		if ( $payload1['door1'] != $payload2['door1'] ) {
			$message .= 'door 1 was '.(($payload1['door1'] == 1)? 'opened' : 'closed' ).' ';
		}
		
		if ( $payload1['door2'] != $payload2['door2'] ) {
			$message .= 'door 2 was '.(($payload1['door2'] == 1)? 'opened' : 'closed' );
		}			
		
		return $message;
	}
}