<?php

namespace Acme\SosMonBundle\Model;
use \DateTime;
class Common
{
	
	static public function object_to_array ( $obj ) 
	{
		if(is_object($obj)) $obj = (array) $obj;
		if(is_array($obj)) {
			$new = array();
			foreach($obj as $key => $val) {
				$new[$key] = Common::object_to_array($val);
			}
		}
		else $new = $obj;
		return $new; 
	}	
	
	/**
	XML safe process
	@param string XML
	@return array
	*/
	
	static public function processXML( $postText )
	{
		libxml_use_internal_errors(true);
		
		$xmlData = simplexml_load_string($postText);
		
		libxml_use_internal_errors(false);	
		return $xmlData;
	}
	
	/**
	XML safe process
	@param string XML
	@return string Json(array)
	*/
	
	static public function processXMLToJSON( $postText )
	{
		libxml_use_internal_errors(true);
		
		$xmlData = simplexml_load_string($postText);
		
		libxml_use_internal_errors(false);	
		return json_encode( $xmlData );
	}	
	
	/**
	Check if array assoc or seq
	@param array $array
	@return bool
	*/	
	static public function is_assoc(array $array) {
		return (bool)count(array_filter(array_keys($array), 'is_string'));
	}	


	/**
	* Predefined array of unit statuses
	*
	* @return array access types
	**/
	static public function getUnitStatuses($view = 0, $opt = 0)
	{	
		if ($view == 0) {
			$list = array( 	
							1 => 'ON',
							2 => 'OFF',
							3 => 'Alert',
							4 => 'NO RESP',
							5 => 'Valid key',
					);
		}
		
		if ($view == 1) {
			$list = array();
			if ($opt == 1) {
				$list[] = array('id' => 0, 'name' => '< ALL >');			
			}
			$list[] = array('id' => 1, 'name' => 'ON');
			$list[] = array('id' => 2, 'name' => 'OFF');
			$list[] = array('id' => 3, 'name' => 'Alert');
			$list[] = array('id' => 4, 'name' => 'NO RESP');
			$list[] = array('id' => 5, 'name' => 'Valid key');
		}		
		return $list;
	}
	
	static public function getStatusName( $id )
	{	
		$result = '';
		$stat = Common::getUnitStatuses();
		$id = (int)$id;
		if (($id >= 0) && ($id < count($stat) )) {
			$result = isset($stat[ $id ]) ? $stat[ $id ] : '';
		}
		
		return $result;
	}	
	
	
	

	static public function getAccessArray($opt = 0)
	{	
		return array(	'users'		=> ($opt == 0 ? 0 : 0),  
						'contacts'	=> ($opt == 0 ? 0 : 1),
						'keyholders'=> ($opt == 0 ? 0 : 2),
						'ibuttons'	=> ($opt == 0 ? 0 : 3),
						'stations'  => ($opt == 0 ? 0 : 4),
						'events'    => ($opt == 0 ? 0 : 5),
						'admin'     => ($opt == 0 ? 0 : 6),
						'call'      => ($opt == 0 ? 0 : 7),
						'loghistory'=> ($opt == 0 ? 0 : 8),
                        'logsummary'=> ($opt == 0 ? 0 : 9),
                        'logcalllog'=> ($opt == 0 ? 0 : 10),						
				);
	}			
	
	static public function getMarkerType() 
	{
				// which status => which marker
		return array(0 => 0,
					 1 => 0,
					 2 => 0,
					 3 => 1,
					 4 => 2,
					 5 => 3,
					 6 => 3,
					 7 => 4,
					 8 => 5,
					 );	
	}
	
	static public function fmtDate( $value )
	{
		$result = '< NONE >';
		if ($value instanceof DateTime) {
			$result = $value->format('m/d/Y');
		}
  	
		return $result;
	}
	
	static public function fmtDateTime( $value )
	{
		$result = '< NONE >';
		if ($value instanceof DateTime) {
			$result = $value->format('m/d/Y H:i:s');
		}
  	
		return $result;
	}
	
	static public function zfdate($orig, $from, $to) 
	{
		$result = '';
		$y = 0;
		$m = 0;
		$d = 0;
		
		if (strlen($orig) > 6) {
		
			if ($from == 'us') {
				$offs = 0;
				if ($orig[1] == '/') { 
					$m = '0'.$orig[0]; 
					$offs++; 
				} else {
					$m = $orig[0].$orig[1]; 
				}
				if ($orig[4 - $offs] == '/') { 
					$d = '0'.$orig[3-$offs]; 
					$offs++; 
				} else { 
					$d = $orig[3-$offs].$orig[4-$offs]; 
				}
				$y = intval($orig[6-$offs].$orig[7-$offs].$orig[8-$offs].$orig[9-$offs]);
			}
		
			if ($from == 'mysql') {
				$y = intval($orig[0].$orig[1].$orig[2].$orig[3]);
				$m = $orig[5].$orig[6];
				$d = $orig[8].$orig[9];
			}	
			if ($from == 'sql') {
				$y = intval($orig[0].$orig[1].$orig[2].$orig[3]);
				$m = $orig[4].$orig[5];
				$d = $orig[6].$orig[7];
			}			
			if($y > 2000) {
				if ($to == 'psql') {  $result = "{$y}-{$m}-{$d}"; }
				if ($to == 'us')   {  $result = "{$m}/{$d}/{$y}"; }
				if ($to == 'date') {  $result = new DateTime("{$y}-{$m}-{$d}"); }
				if ($to == 'mysql') { $result = "{$y}{$m}{$d}"; }
			}
		}
	
		return $result;
	}

	static public function detectZoom( $value ) 
	{
		$zoom = 10;
		if ($value < 0.04) {
			$markers['zoom']     = 13;
		}	
		if (($value < 0.09) && ($value >= 0.04)) {
			$zoom     = 12;
		}
		if (($value < 0.16) && ($value >= 0.09)) {
			$zoom     = 11;
		}		
		if (($value < 0.4) && ($value >= 0.16)) {
			$zoom     = 10;
		}
		if (($value < 1) && ($value >= 0.5)) {
			$zoom     = 9;
		}		
		if (($value < 2) && ($value >= 1)) {
			$zoom     = 8;
		}
		if (($value < 3) && ($value >= 2)) {
			$zoom     = 7;
		}
		if (($value < 4) && ($value >= 3)) {
			$zoom     = 6;
		}	
		if (($value < 5) && ($value >= 4)) {
			$zoom     = 5;
		}		
		if ($value > 5) {
			$zoom     = 4;
		}

		return $zoom;
	}
/*
	Converting Decimal to DMS
	@param float $decimal
	@param bool $type (lat/long)
*/
	static public function DecimalToDMS($decimal, $type = true) {
     //set default values for variables passed by reference
		$degrees = 0;
		$minutes = 0;
		$seconds = 0;
		$direction = 'X';
 
     //decimal must be integer or float no larger than 180;
     //type must be Boolean
     if(!is_numeric($decimal) || abs($decimal) > 180 || !is_bool($type)) {
          return '';
     }
      
     //inputs OK, proceed
     //type is latitude when true, longitude when false
      
     //set direction; north assumed
     if(!$type && $decimal < 0) { 
          $direction = 'S';
     }
     elseif($type && $decimal < 0) {
          $direction = 'W';
     }
     elseif($type) {
          $direction = 'E';
     }
     else {
          $direction = 'N';
     }
     /* 
     //get absolute value of decimal
     $d = abs($decimal);
      
     //get degrees
     $degrees = floor($d);
      
     //get seconds
     $seconds = ($d - $degrees) * 3600;
      
     //get minutes
     $minutes = floor($seconds / 60);
      
     //reset seconds
     $seconds = floor($seconds - ($minutes * 60));  
	 //decimal degrees: 40.446 N 79.982 W
	 // \rm{decimal\  degrees} = \rm{degrees} + \rm{minutes}/60 + \rm{seconds}/3600.
	 */
	 $result = ''.round(abs($decimal),3).'&deg; '.$direction;
	 return $result;
}
	
	/**
	* Predefined array of access types
	*
	* @return array access types
	**/
	static public function getAccessTypes()
	{	
		return array(   1 => 'Administrator',
						2 => 'User',
						3 => 'Alert contact'
					);
	}	
}