<?php
namespace Acme\SosMonBundle\Model;  
use Acme\SosMonBundle\Event\PackageEvent; 
use Acme\SosMonBundle\Event\CampaignEvent; 
use Acme\SosMonBundle\Model\Payload;
use \DateTime;
 
class PackageProcessing
{
	private $em;
	private $dispatcher;
	private $logger;
	
	public function __construct($em, $dispatcher, $logger) {
		$this->em   = $em;
		$this->dispatcher = $dispatcher;
		$this->logger = $logger;
	} 
 
 /*
    OnPackage Event Processing
	
 */
    public function onPackageEvent(PackageEvent $event) 
	{
		$message = $event->getMessage();
		
		$success   = 1;	
		$msgCount  = 0;
	
		$amsg = json_decode($message, TRUE);

		$messageid = $amsg['@attributes']['messageID'];

	// save a history	
		$rawid = $this->em->getRepository('AcmeSosMonBundle:Raw')->
						saveEntity(array(   'message' => $message, 
											'success' => 0,
											'messageid' => $messageid,
								   ));	

		$result = array( 'eventid' => 0,
						 'esn' 	   => '',
						 'rawid'   => $rawid,
		);						   
		
	// processing events	

		if ( isset($amsg['stuMessage']) ) { // at least 1 element exists
			if ( Common::is_assoc( $amsg['stuMessage'] ) == false ) {// case of more then 1 message at package

				foreach ($amsg['stuMessage'] as $m) {   
					$result = $this->proceedMessage( $rawid, $m );
					$msgCount++;
					if (($result['eventid'] == 0) && ($result['esn'] != '00-00000')) {
						$success = 0; // At least one fail means all fail
					}			
				}
			}		
			else { // case 1 message at package		
				$result = $this->proceedMessage( $rawid, $amsg['stuMessage'] );
				$msgCount++;
				if (($result['eventid'] == 0) && ($result['esn'] != '00-00000')) {
					$success = 0; // At least one fail means all fail	
				}
			}
		}
		$success   = 1;
		
		$this->em->getRepository('AcmeSosMonBundle:Raw')->setFinalize($rawid, $success);	
			
		$pass = ($success) ? 'pass' : 'fail';
		$note = ($success) ? $msgCount.' messages stored' : '';
		
		return $this->sendResponseMessage( $messageid, $pass, $note );
    }
	
	/*
	** proceedMessage
	**
	** @param rawid integer
	** @param string msg 
	** @return array('eventid'  => 0,
				  	 'esn' 	    => '',
					 'rawid'    => $rawid,
					 'stationid'=> 0,
		)
	*/
	public function proceedMessage( $rawid, $msg )
	{
		$result = array( 'eventid' => 0,
						 'esn' 	   => '',
						 'rawid'   => $rawid,
						 'stationid'=> 0,
		);
		
		$rep = $this->em->getRepository('AcmeSosMonBundle:Event');
					
		$result['esn'] =  isset($msg['esn']) ? (string)$msg['esn'] : 'failed';
		$station = $this->em->getRepository('AcmeSosMonBundle:Station')->findOneByEsn( $result['esn'] );
		
		if ( $station ) {
			
			$date   = new DateTime();		
			$event  = $rep->findEntity(0, 0);
			
			$event['rawid'] 		= $rawid;		
			$event['ownid'] 		= $station->getOwnid();
			$event['stationid'] 	= $station->getId();
			$event['status']		= $station->getStatus();
			$event['ibuttonid']		= -1;
			$event['keyholderid']	= -1;
			$event['response']	    = 0;
			$event['latitude'] 		= $station->getLatitude();
			$event['longitude']		= $station->getLongitude();
			$event['charge'] 		= ($station->getCharge() < 3) ? 100 : $station->getCharge() - 1;
			$event['edata'] 		= isset($msg['payload']) ? (string) $msg['payload'] : '';
			$event['dt']            = Common::fmtDateTime( $date->setTimestamp( (int)$msg['unixTime'] ) );	
			
			$payload = Payload::Decode9byte( $event['edata'] );
			
			$iB = $this->em->getRepository('AcmeSosMonBundle:Ibutton')->findOneByCrc($payload['crc']);
			if (is_object($iB)) {
				$event['ibuttonid'] = $iB->getId();
			}			
			
/*  STATUS
							1 => 'ON',
							2 => 'OFF',
							3 => 'Alert',
							4 => 'NO RESP',
							5 => valid key

	At this point : issys = true - system message - hold a lat-long values
*/
			
			if ( $payload['issys'] == false ) {
				if ( $payload['pir'] == 1) {
					$event['status'] = 3; // pir alert
				}
				else {
					$event['status'] = $this->checkCRC( $event['stationid'], $payload['crc'] );			
				}
			}
			else {
				$event['status'] = 1;
			}
			
			if ( $payload['isgps'] ) {
				$event['latitude'] 		= $payload['lat'];
				$event['longitude']		= $payload['long'];			
			}
			
//echo print_r($event);										
			if ( ($event['ownid'] > 0) && ($event['stationid'] > 0) && ($event['status'] > 0) ) {	
				// create new event

				$result['eventid'] = $rep->saveEntity( $event ); 
				$rep->saveResponse( $result['eventid'], 0, 0 );
			
				// upgrade station status
				$station->setLatitude ( $event['latitude'] )
						->setLongitude( $event['longitude'] )	
						->setCharge   ( (int)$event['charge'] )
						->setEventid  ( $result['eventid'] );
						
				if ( $payload['issys'] == false ) {
					$station->setStatus( (int)$event['status'] );
				}
				
				$this->em->getRepository('AcmeSosMonBundle:Station')->commitEntity( $station );	
				
// ******  ALERT EVENT RAISE HERE				
				if ($event['status'] == 3) {					
					$this->dispatcher->dispatch('acme.events.startcampaign', new CampaignEvent( $rep->find( $result['eventid'] ) ));				
				}	
			}
			
		}
		
		return $result;
	}
	
	public function checkCRC( $stationid, $crc )
	{
		return ( $this->em->getRepository('AcmeSosMonBundle:Keystations')->getCrcstations($crc, $stationid) == true) ? 5 : 3;
	}
	
	
	
	public function sendResponseMessage( $messageid, $passed, $stateMessage )
	{
		$input_xml = GlobalstarConfig::getstuResponseMsg( $messageid, $passed, $stateMessage );

		if (strlen(GlobalstarConfig::getUrl()) > 0) {
			$data = '';
			
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, GlobalstarConfig::getUrl() );
// Following line is compulsary to add as it is:
			curl_setopt($ch, CURLOPT_POSTFIELDS, $input_xml);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 300);
			$data = curl_exec($ch);
			curl_close($ch);
		}
		return $input_xml;
	}
	
	public function testMessage( $esn, $payload) 
	{
		return '<?xml version="1.0" encoding="UTF-8"?>
            <stuMessages timeStamp="16/07/2015 23:18:15 GMT"
            messageID="56bdca48088610048fddba385e1cd5b8">
			<stuMessage><esn>'.$esn.'</esn><unixTime>'.time().'</unixTime><gps>N</gps><payload>'.$payload.'</payload></stuMessage>
			</stuMessages>';	
	}
	
/*



*/	
}