<?php

namespace Acme\SosMonBundle\Tests\Controller;

use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;
use Acme\SosMonBundle\Entity\Contact;

class ContactControllerTest extends WebTestCase
{
	private $ownid = 0;
	public function testModel() 
	{
		$kernel = static::createKernel();
		$kernel->boot();
		$em = $kernel->getContainer()->get('doctrine.orm.entity_manager');	

		$reps = $em->getRepository('AcmeSosMonBundle:Contact');
		$this->ownid = $em->getRepository('AcmeSosMonBundle:Customer')->getTestOwnid();
		
		//save entity
		// add new
		$id = $reps->saveEntity(  array( 
									'id'       => 0,
									'ownid'    => $this->ownid,
									'fname'    => 'John',
									'lname'    => 'Doe',
									'phone1'   => '1',
									'phone2'   => '2',
									'phone3'   => '3',
									'phone4'   => '4',
									'sms' 	   => 'sms',
									'email'    => 'ee',
									'priority' => 1,
								));
		$this->assertTrue($id > 0);	

		$id = $reps->saveEntity(  array( 
									'id'       => $id,
									'ownid'    => $this->ownid,
									'fname'    => 'Ilya',
									'lname'    => 'Kaurov',
									'phone1'   => '11',
									'phone2'   => '22',
									'phone3'   => '33',
									'phone4'   => '44',
									'sms' 	   => '55',
									'email'    => 'ee@ee.ee',
									'priority' => 1,
								));		
		$entity = $reps->findEntity( $id, $this->ownid );	
		
		$this->assertTrue($entity['id']    == $id);	
		$this->assertTrue($entity['ownid'] == $this->ownid);	
		$this->assertTrue($entity['priority']	== 1);
		$this->assertTrue($entity['phone1']     == '11');
		$this->assertTrue($entity['phone2']     == '22');
		$this->assertTrue($entity['phone3']     == '33');
		$this->assertTrue($entity['phone4']     == '44');		
		$this->assertTrue($entity['sms']   		== '55');
		$this->assertTrue($entity['email']      == 'ee@ee.ee');	
		
		$dbl = $reps->checkPhone(  array( 
									'id'       => 0,
									'ownid'    => $this->ownid,
									'fname'    => 'Ilya',
									'lname'    => 'Kaurov',
									'phone1'   => '11',
									'phone2'   => '22',
									'phone3'   => '33',
									'phone4'   => '44',
									'sms' 	   => '55',
									'email'    => 'ee@ee.ee',
									'priority' => 1,
								));
		
		$this->assertFalse($dbl);	
		
		$list = $reps->getList( 1, 2 );
		$this->assertTrue(count($list) > 0);   // at least one customer exists		
		$reps->deleteEntity( $id );
	}

/*
	index route - must show login form
*/		
	public function testIndex()
	{
        $client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '222',
			));
        $crawler = $client->request('GET', '/app/contacts');
        $this->assertTrue($crawler->filter('html:contains("Alarm")')->count() > 0);	
	}
	
/*
	users_data route - must show login form
*/	
	public function testData()
	{
		$client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '222',
			));

		$client->request('GET', '/app/contacts_data');
		$this->assertTrue(is_array(json_decode( $client->getResponse()->getContent() )));

	}	
	
/*
	entity route 
*/	
	public function testEntity()
	{
		$client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '222',
			));

		$kernel = static::createKernel();
		$kernel->boot();
		$em = $kernel->getContainer()->get('doctrine.orm.entity_manager');	
		
		$reps = $em->getRepository('AcmeSosMonBundle:Contact');
		
		//save entity
		// add new
		$id = $reps->saveEntity(  array( 
									'id'       => 0,
									'ownid'    => $this->ownid,
									'fname'    => 'John',
									'lname'    => 'Doe',
									'phone1'   => '1',
									'phone2'   => '2',
									'phone3'   => '3',
									'phone4'   => '4',
									'sms' 	   => 'sms',
									'email'    => 'ee',
									'priority' => 1,
								));
		$this->assertTrue($id > 0);			
			
		$crawler = $client->request('GET', '/app/contacts_entity/'.$id);
		$this->assertTrue($crawler->filter('html:contains("Email")')->count() > 0);
		
		$crawler = $client->request('GET', '/app/contacts_id/'.$id);
		$this->assertTrue(is_object(json_decode( $client->getResponse()->getContent() )));
		
		$client->request('GET', '/app/contacts_delete/'.$id);
		$obj = json_decode($client->getResponse()->getContent());
		$this->assertTrue($obj->success);		
		
	}		
}