<?php

namespace Acme\SosMonBundle\Tests\Controller;

use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;
use Acme\SosMonBundle\Entity\Customer;

class CustomerControllerTest extends WebTestCase
{

	public function testModel() 
	{
		$kernel = static::createKernel();
		$kernel->boot();
		$em = $kernel->getContainer()->get('doctrine.orm.entity_manager');	
		
		$em->getConnection()->executeUpdate('delete from customers where prefix = :dummy or prefix = :pref', 
			array('dummy' => 'dummy',
			      'pref'  => 'pref')
		);


		$reps = $em->getRepository('AcmeSosMonBundle:Customer');
		
		$this->assertTrue( $reps->checkPrefix('pref') );
		
		//save entity
		// add new
		$id = $reps->saveEntity(  array( 'ownid'   => 0,
									'name'    => 'Test company',
									'shipping'  => '1',
									'billing'   => '2',
									'contact_name'    => '3',
									'contact_pos'     => '4',
									'contact_phone'   => '5',
									'contact_mobile'  => '6',
									'contact_email'   => '7',							
									'keys' 	  => 0,
									'prefix'  => 'pref',
									'sentry'  => 0,
									'deflat'  => 40,
									'deflong' => 50,
									'rrate'   => 0,
									'active'  => 0,
									'mstype'  => 0,
									'billcontact'=> '',						
									'fixedcount' 	=> 0,
									'fixedprice' 	=> 0,
									'extraprice' 	=> 0,									
								));
		$this->assertTrue($id > 0);			

		$this->assertTrue( $reps->checkPrefix( 'pref') == false );

		$id = $reps->saveEntity(  array( 'ownid'   => $id,
									'name'    => 'Test',
									'shipping'  => '11',
									'billing'   => '22',
									'contact_name'    => '33',
									'contact_pos'     => '44',
									'contact_phone'   => '55',
									'contact_mobile'  => '66',
									'contact_email'   => '77',										
									'keys' 	  => 1,
									'prefix'  => 'pref222',									
									'sentry'  => 1,
									'deflat'  => 40.14,
									'deflong' => 50.55,
									'rrate'   => 1,
									'active'  => 1,
									'mstype'  => 1,
									'billcontact'=> 'bc',						
									'fixedcount' 	=> 100,
									'fixedprice' 	=> 10.05,
									'extraprice' 	=> 0.02,									
								));		
		$entity = $reps->findEntity( $id );																				
		
		$this->assertTrue($entity['ownid']    == $id);	
		$this->assertTrue($entity['name']	  == 'Test');
		$this->assertTrue($entity['shipping']	      == '11');		
		$this->assertTrue($entity['billing']	      == '22');
		$this->assertTrue($entity['contact_name']	  == '33');
		$this->assertTrue($entity['contact_pos']	  == '44');
		$this->assertTrue($entity['contact_phone']	  == '55');
		$this->assertTrue($entity['contact_mobile']	  == '66');
		$this->assertTrue($entity['contact_email']	  == '77');
		$this->assertTrue($entity['prefix']	  == 'pref');
		$this->assertTrue($entity['keys']     == 1);
		$this->assertTrue($entity['sentry']   == 1);
		$this->assertTrue($entity['rrate']    == 1);	
		$this->assertTrue($entity['active']   == 1);	
		$this->assertTrue($entity['mstype']   == 1);
		$this->assertTrue($entity['deflat']   == 40.14);
		$this->assertTrue($entity['deflong']  == 50.55);
		$this->assertTrue($entity['billcontact']  == 'bc');		
		$this->assertTrue($entity['fixedcount']  == 100);		
		$this->assertTrue($entity['fixedprice']  == 10.05);		
		$this->assertTrue($entity['extraprice']  == 0.02);		
		
		$reps->activeEntity( $id );
		$list = $reps->getList();
		$this->assertTrue(count($list) > 0);   // at least one customer exists
		foreach($list as $rs) {
			if ($rs['ownid'] == $id) {
				$this->assertTrue($rs['active'] == 0);
			}
		}
		
		$reps->deleteEntity( $id );

		$list = $reps->getList();
		$this->assertTrue(count($list) > 0);   // at least one customer exists
		foreach($list as $rs) {
			if ($rs['ownid'] == $id) {
				$this->assertTrue( 1 == 0 );
			}
		}
		
		$this->assertTrue($reps->getTestOwnid() > 0);
	}

/*
	index route - must show login form
*/		
	public function testIndex()
	{
		$client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '333333',
			));
			
        $crawler = $client->request('GET', '/app/customers');
		
        $this->assertTrue($crawler->filter('html:contains("Customers")')->count() > 0);	
	}	
/*
	users_data route - must show login form
*/	
	public function testData()
	{
		$client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '333333',
			));

		$client->request('GET', '/app/customers_data');
		$this->assertTrue(is_array(json_decode( $client->getResponse()->getContent() )));
	}	
	
/*
	entity route 
*/	
	public function testEntity()
	{
		$client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '333333',
			));

		$kernel = static::createKernel();
		$kernel->boot();
		$em = $kernel->getContainer()->get('doctrine.orm.entity_manager');			
		$reps = $em->getRepository('AcmeSosMonBundle:Customer');
		$list = current($reps->getList());
		
		$crawler = $client->request('GET', '/app/settings/'.$list['ownid']);
		$this->assertTrue($crawler->filter('html:contains("Sentry")')->count() > 0);
	}	
	
/*
	activate route 
*/	
	public function testActive()
	{
		$client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '333333',
			));

		$client->request('GET', '/app/customers_active/66');
		$obj = json_decode($client->getResponse()->getContent());
		$this->assertTrue($obj->success);

	}

/*
	checkprefix route 
*/	
	public function testCheckprefix()
	{
		$client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '333333',
			));

		$client->request('GET', '/app/customers_checkprefix/dummy');
		$obj = json_decode($client->getResponse()->getContent());
		
		$this->assertTrue($obj->success);
		
		$kernel = static::createKernel();
		$kernel->boot();
		$em = $kernel->getContainer()->get('doctrine.orm.entity_manager');			
		$reps = $em->getRepository('AcmeSosMonBundle:Customer');		
		$reps->deleteEntity( $obj->id );
	}
	
/*
	delete route 
*/	
	public function testDelete()
	{
		$client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '333333',
			));
			
		$kernel = static::createKernel();
		$kernel->boot();
		$em = $kernel->getContainer()->get('doctrine.orm.entity_manager');			
		$reps = $em->getRepository('AcmeSosMonBundle:Customer');
		// add new
		$id = $reps->saveEntity(  array( 'ownid'   => 0,
									'name'    => 'Test company',
									'shipping'  => '1',
									'billing'   => '2',
									'contact_name'    => '3',
									'contact_pos'     => '4',
									'contact_phone'   => '5',
									'contact_mobile'  => '6',
									'contact_email'   => '7',							
									'keys' 	  => 0,
									'prefix'  => 'pref',
									'sentry'  => 0,
									'deflat'  => 40,
									'deflong' => 50,
									'rrate'   => 0,
									'active'  => 0,
									'mstype'  => 0,
									'billcontact'=> 'bc',						
									'fixedcount' 	=> 100,
									'fixedprice' 	=> 10.05,
									'extraprice' 	=> 0.02,									
								));			

		$client->request('GET', '/app/customers_delete/'.$id);
		$obj = json_decode($client->getResponse()->getContent());
		$this->assertTrue($obj->success);
		
		$em->getConnection()->executeUpdate('delete from customers where prefix = :dummy or prefix = :pref', 
			array('dummy' => 'dummy',
			      'pref'  => 'pref')
		);		
	}	
}