<?php

namespace Acme\SosMonBundle\Tests\Controller;

use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;
use Acme\SosMonBundle\Entity\Event;
use Acme\SosMonBundle\Model\Common;
use \DateTime;

class EventControllerTest extends WebTestCase
{
	private $em;
	private $reps;
	
	private $ownid;
	private $eventid = 0;
	
	public function __construct() {
		$kernel = static::createKernel();
		$kernel->boot();
		$this->em    = $kernel->getContainer()->get('doctrine.orm.entity_manager');
		$this->reps  = $this->em->getRepository('AcmeSosMonBundle:Event');	
		$this->ownid = $this->em->getRepository('AcmeSosMonBundle:Customer')->getTestOwnid();		
	}	

	public function testModel() 
	{
//		$kernel = static::createKernel();
//		$kernel->boot();
//		$this->em = $kernel->getContainer()->get('doctrine.orm.entity_manager');	

//		$this->reps = $em->getRepository('AcmeSosMonBundle:Event');
		
		//save entity
		// add new
//		m/d/Y H:i:s
		$d1 = '01/01/2015 03:15:34';
		$this->eventid = $this->reps->saveEntity(  array( 
				'id' 			=> 0,			
				'ownid'    		=> -2,
				'stationid' 	=> -1,	
				'ibuttonid' 	=> -1,			
				'keyholderid' 	=> -1,
				'status'		=> 1,
				'longitude' 	=> 1,				
				'latitude' 		=> 2,	
				'rawid'         => 0,
				'charge'		=> 10,		
				'edata' 		=> 'ed',
				'dt' 			=> $d1,
								));
					
		$this->assertTrue($this->eventid > 0);	

		$this->reps->saveEntity(  array( 
				'id' 			=> $this->eventid,			
				'ownid'    		=> $this->ownid,
				'stationid' 	=> 1,	
				'ibuttonid' 	=> 2,			
				'keyholderid' 	=> 3,
				'status'		=> 2,
				'longitude' 	=> 100,	
				'rawid'         => 2,				
				'latitude' 		=> 110,	
				'charge'		=> 34,		
				'edata' 		=> 'edata',
				'dt' 			=> $d1,
								));		
		$entity = $this->reps->findEntity( $this->eventid, $this->ownid );																				
		
		$this->assertTrue($entity['id']    == $this->eventid);	
		$this->assertTrue($entity['ownid'] == $this->ownid);		
		$this->assertTrue($entity['stationid']     == 1);
		$this->assertTrue($entity['ibuttonid']     == 2);
		$this->assertTrue($entity['keyholderid']   == 3);
	//	$this->assertTrue($entity['dt']     == $d1);
		$this->assertTrue($entity['status']        == 2);
		$this->assertTrue($entity['rawid']        == 2);
		$this->assertTrue($entity['longitude']     == 100);
		$this->assertTrue($entity['latitude']      == 110);
		$this->assertTrue($entity['charge']        == 34);
		$this->assertTrue($entity['edata']         == 'edata');


		
		// find test
		$entity = $this->reps->findEntity( $this->eventid, $this->ownid );	
		$this->assertTrue($entity['id']    == $this->eventid);	
		
		$id = -1;
		$entity = $this->reps->findEntity( $id, $this->ownid );	
		$this->assertTrue($entity['id']  != $id);	
		
		$this->reps->deleteEntity( $this->eventid )	;

		$entity = $this->reps->findEntity( $this->eventid, $this->ownid );	
		$this->assertTrue($entity['id']    == 0);		
	}
	
	public function testNotifiers()
	{	
		$this->eventid = $this->reps->saveEntity(  array( 
				'id' 			=> 0,			
				'ownid'    		=> -2,
				'stationid' 	=> -1,	
				'ibuttonid' 	=> -1,			
				'keyholderid' 	=> -1,
				'status'		=> 1,
				'longitude' 	=> 1,				
				'latitude' 		=> 2,	
				'rawid'         => 0,
				'charge'		=> 10,		
				'edata' 		=> 'ed',
				'dt' 			=> '01/01/2015 03:15:34',
								));
		
		$list = $this->reps->getNotifies( $this->eventid, $this->ownid );
		$this->assertTrue(is_array($list));
		$this->reps->deleteEntity( $this->eventid )	;	
		$list = $this->reps->getNotifies( -1, $this->ownid );
		$this->assertTrue(is_array($list));		
	}	
	
	public function testList()
	{
		
		$dt1         = Common::zfdate(str_replace(".", "/", '01/01/2015'), 'us','mysql');
		$dt2         = Common::zfdate(str_replace(".", "/", '01/25/2015'), 'us','mysql');
		
		$list = $this->reps->getList( 0,0, 1, -1, -1, -1, -1, 0, $dt1, $dt2 );
		$this->assertTrue(is_array($list));
			
	}	
	
	public function testAlert()
	{	
		$dt1         = Common::zfdate(str_replace(".", "/", '01/01/2015'), 'us','mysql');
		$dt2         = Common::zfdate(str_replace(".", "/", '01/25/2015'), 'us','mysql');
		
		$list = $this->reps->getAlerts( 1, $dt1, $dt2 );
		$this->assertTrue(is_array($list));
	}	
		
	
	public function testResponse()
	{
		$this->eventid = $this->reps->saveEntity(  array( 
				'id' 			=> 0,			
				'ownid'    		=> -2,
				'stationid' 	=> -1,	
				'ibuttonid' 	=> -1,			
				'keyholderid' 	=> -1,
				'status'		=> 1,
				'longitude' 	=> 1,				
				'latitude' 		=> 2,	
				'rawid'         => 0,
				'charge'		=> 10,		
				'edata' 		=> 'ed',
				'dt' 			=> '01/01/2015 03:15:34',
								));	
	
		$entity = $this->reps->findEntity( $this->eventid, $this->ownid );
		$respond = (int)$entity['response'];
		$respond++;
		
		$this->reps->saveResponse($this->eventid, $respond);
		$entity = $this->reps->findEntity( $this->eventid, $this->ownid );
				
		$this->assertTrue($entity['response'] == $respond);
		
		$this->reps->saveResponse($this->eventid, 'error');
		$entity = $this->reps->findEntity( $this->eventid, $this->ownid );
		$this->assertTrue($entity['response'] == 0);
		
		$this->reps->deleteEntity( $this->eventid );		
	}
	
	public function testeventDescription()
	{
	
		$this->eventid = $this->reps->saveEntity(  array( 
				'id' 			=> 0,			
				'ownid'    		=> -2,
				'stationid' 	=> -1,	
				'ibuttonid' 	=> -1,			
				'keyholderid' 	=> -1,
				'status'		=> 1,
				'longitude' 	=> 1,				
				'latitude' 		=> 2,	
				'rawid'         => 0,
				'charge'		=> 10,		
				'edata' 		=> 'ed',
				'dt' 			=> '01/01/2015 03:15:34',
								));
								
		$desc = $this->reps->eventDescription(-1);
		$this->assertTrue($desc == '< None >');
		
		$desc = $this->reps->eventDescription($this->eventid);

		$this->assertTrue($desc != '< None >');
		$this->reps->deleteEntity( $this->eventid );
	}	
	
	
	public function testgetMarkersForEvents()
	{
		
		$dt1         = Common::zfdate(str_replace(".", "/", '01/01/2015'), 'us','mysql');
		$dt2         = Common::zfdate(str_replace(".", "/", '01/25/2015'), 'us','mysql');
		
		$markers = $this->reps->getMarkersForEvents(10,0,0,0,0,0, $dt1, $dt2, -90, 45);
          		
		$this->assertTrue(is_array($markers));
			
	}
	
	
	public function testDelete()
	{
	
		$this->eventid = $this->reps->saveEntity(  array( 
				'id' 			=> 0,			
				'ownid'    		=> -2,
				'stationid' 	=> -1,	
				'ibuttonid' 	=> -1,			
				'keyholderid' 	=> -1,
				'status'		=> 1,
				'longitude' 	=> 1,				
				'latitude' 		=> 2,	
				'rawid'         => 0,
				'charge'		=> 10,		
				'edata' 		=> 'ed',
				'dt' 			=> '01/01/2015 03:15:34',
								));	
								
		// delete test
		$this->assertTrue( $this->reps->deleteEntity( $this->eventid ) );
		
		$entity = $this->reps->findEntity( $this->eventid, $this->ownid );
		$this->assertTrue($entity['id'] != $this->eventid);
		
		$this->assertTrue( $this->reps->deleteEntity( -1 ) == false );		
	}
	
			
}