<?php

namespace Acme\SosMonBundle\Tests\Controller;

use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;
use Acme\SosMonBundle\Entity\Ibutton;

class IbuttonControllerTest extends WebTestCase
{

	public function testModel() 
	{
		$kernel = static::createKernel();
		$kernel->boot();
		$em = $kernel->getContainer()->get('doctrine.orm.entity_manager');	
		$ownid = $em->getRepository('AcmeSosMonBundle:Customer')->getTestOwnid();
		
		$reps = $em->getRepository('AcmeSosMonBundle:Ibutton');
		
		//save entity
		// add new
		$id = $reps->saveEntity(  array( 
									'id'       => 0,
									'ownid'    => $ownid,
									'keyholderid' => 0,
									'key'      => 'ABCD',
									'active'   => 0,
									'crc' => '12',
								));
		$this->assertTrue($id > 0);	

		$id = $reps->saveEntity(  array( 
									'id'       => $id,
									'ownid'    => $ownid,
									'keyholderid' => 0,
									'key'      => 'ABCDEF',
									'active'   => 1,
									'crc' => '13',
								));	
		$entity = $reps->findEntity( $id, $ownid );																				
		
		$this->assertTrue($entity['id']    == $id);	
		$this->assertTrue($entity['ownid'] == $ownid);	
		$this->assertTrue($entity['key']   == 'ABCDEF');
		$this->assertTrue($entity['crc']   == '13');
		$this->assertTrue($entity['active']== 1);

		$list = $reps->getAvailableIbuttons( $ownid, 0 );
		$this->assertTrue(is_array( $list ));
		
		$this->assertTrue( $em->getRepository('AcmeSosMonBundle:Keyholder')->assignButton(0, $id) == 1 );
		$reps->activeEntity( $id );

		$list = $reps->getList( $ownid, 2, '' );		
		$this->assertTrue(count($list) > 0);   // at least one customer exists		
		$rid = $em->getRepository('AcmeSosMonBundle:Keyholder')->assignButton( $id, 1);	
		$this->assertTrue($rid == 1);   // at least one customer exists	

		$entity = $reps->findEntity( $id, $ownid );
		$this->assertTrue( $entity['active']== 0);		
		
		$reps->deleteEntity( $id );
	}

/*
	index route - must show login form
*/		
	public function testIndex()
	{
        $client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '222',
			));
        $crawler = $client->request('GET', '/app/ibuttons');
		
        $this->assertTrue($crawler->filter('html:contains("Ibuttons")')->count() > 0);	
	}
	
/*
	users_data route - must show login form
*/	
	public function testData()
	{
		$client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '222',
			));

		$client->request('GET', '/app/ibuttons_data');
		$this->assertTrue(is_array(json_decode( $client->getResponse()->getContent() )));

	}	
	
/*
	entity route 
*/	
	public function testEntity()
	{
		$client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '222',
			));
			
		$kernel = static::createKernel();
		$kernel->boot();
		$em = $kernel->getContainer()->get('doctrine.orm.entity_manager');	
		$ownid = $em->getRepository('AcmeSosMonBundle:Customer')->getTestOwnid();
		
		$reps = $em->getRepository('AcmeSosMonBundle:Ibutton');
		
		//save entity
		// add new
		$id = $reps->saveEntity(  array( 
									'id'       => 0,
									'ownid'    => $ownid,
									'keyholderid' => 0,
									'key'      => 'ABCD',
									'active'   => 0,
									'crc' => '12',
								));			

		$crawler = $client->request('GET', '/app/ibuttons_entity/'.$id);
		$this->assertTrue($crawler->filter('html:contains("Key")')->count() > 0);
		
		$crawler = $client->request('GET', '/app/ibuttons_active/'.$id);
		$entity = $reps->findEntity( $id, $ownid );

		$this->assertTrue( $entity['active']== 0);	

		$crawler = $client->request('GET', '/app/ibuttons_delete/'.$id);
		$obj = json_decode($client->getResponse()->getContent());
		$this->assertTrue($obj->success);		

	}	
	
	
/*
	delete route 
	
	public function testDelete()
	{
		$client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '222',
			));

		$client->request('GET', '/app/customers_delete/1');
		$obj = json_decode($client->getResponse()->getContent());
		$this->assertTrue($obj->success);

	}
*/	
}