<?php

namespace Acme\SosMonBundle\Tests\Controller;

use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;
use Acme\SosMonBundle\Entity\User;

class UserControllerTest extends WebTestCase
{

	public function testModel() 
	{
		$kernel = static::createKernel();
		$kernel->boot();
		$em = $kernel->getContainer()->get('doctrine.orm.entity_manager');	

		$reps = $em->getRepository('AcmeSosMonBundle:User');
		
	
		//save entity
		// add new
		$id = $reps->saveEntity(  array( 'userid'   => 0,
									'name'     => 'Test user',
									'login'    => 'test',
									'password' => '12',
									'phone'    => '111-222',
									'mobile'   => '333-444',
									'email'    => 'ex@ex.com',
									'active'   => 0,
									'accword'  => '000000000000000000000000000',
									'ownid'    => -1,
									'acclevel' => 1,
									'asystems' => '',
									'chgpassword' =>0,
									'alertcontactid' => 0,
								), 0);
		$this->assertTrue($id > 0);	
		//edit
		$id = $reps->saveEntity( array( 'userid'   => $id,
										'name'     => 'Test2',
										'login'    => 'test_user',
										'password' => 'regenbogen',
										'phone'    => '111-222-999',
										'mobile'   => '333-444-888',
										'email'    => 'example@example.com',	
										'active'   => 1,										
										'accword'  => '222222222222222222222',
										'ownid'    => -2,
										'acclevel' => 3,
										'asystems' => '',
										'chgpassword' =>0,
										'alertcontactid' => 0,										
										),0 );
										
		$reps->chgPasswordEntity($id, 1);
		
		
		$user = $reps->findEntity( $id );											
		$res = $reps->checkEntity( 'test_neveruser' ); // good								
	
		$this->assertTrue($res == 1);	
		$res = $reps->checkEntity( 'test_user' ); // bad
		$this->assertTrue($res == 0);		

		$this->assertTrue($user['chgpassword'] == 1);			
		
		$this->assertTrue($user['userid'] == $id);	
		$this->assertTrue($user['name'] == 'Test2');
		$this->assertTrue($user['login'] == 'test_user');
		$this->assertTrue($user['password'] == 'regenbogen');
		$this->assertTrue($user['phone']    == '111-222-999');		
		$this->assertTrue($user['mobile']   == '333-444-888');
		$this->assertTrue($user['accword']  == '222222222222222222222');
		$this->assertTrue($user['email']    == 'example@example.com');
		$this->assertTrue($user['active']   == 1);		
		$this->assertTrue($user['ownid']    == -2);
		$this->assertTrue($user['acclevel'] == 3);
		$this->assertTrue($reps->getSectionAccess($user['accword'], 'users') == 2);
		$this->assertTrue($reps->getSectionAccess($user['accword'], 'noindex') == 0);
		
			
		$reps->activeEntity( $id, 1 );
		$list = $reps->getList(-2, '');
		foreach($list as $rs) {
			if ($rs['userid'] == $id) {
				$this->assertTrue($rs['active'] == 1);
			}
		}
		// getList
		$this->assertTrue(count($reps->getList(-2)) > 0);   // at least one user exists
		$this->assertTrue(count($reps->getList(-1)) == 0); // empty
		$this->assertTrue(count($reps->getAccessTypes()) > 0);
		
		$reps->deleteEntity( $id );
		
	}
/*
	index route - must show login form
*/		
	public function testIndex()
	{
        $client = static::createClient();
        $crawler = $client->request('GET', '/login');
        $this->assertTrue($crawler->filter('html:contains("Authentication")')->count() > 0);	
	}
/*
	users_data route - must show login form
*/	
	public function testData()
	{
		$client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '333333',
			));

		$client->request('GET', '/app/users_data');
		
	//	print_r($client->getResponse()->getContent());
		
		$this->assertTrue(is_array(json_decode( $client->getResponse()->getContent() )));

	}	
	
/*
	entity route 
*/	
	public function testEntity()
	{
		$client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '333333',
			));

		$crawler = $client->request('GET', '/app/users_entity/108');
		$this->assertTrue($crawler->filter('html:contains("Web users")')->count() > 0);

	}	
	
/*
	activate route 
*/	
	public function testActive()
	{
		$client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '333333',
			));

		$client->request('GET', '/app/users_active/66');
		$obj = json_decode($client->getResponse()->getContent());
		$this->assertTrue($obj->success);

	}
	
/*
	delete route 
*/	
	public function testDelete()
	{
		$client = static::createClient(array(), array(
			'PHP_AUTH_USER' => '111',
			'PHP_AUTH_PW'   => '333333',
			));

		$client->request('GET', '/app/users_delete/1');
		$obj = json_decode($client->getResponse()->getContent());
		$this->assertTrue($obj->success);

	}	
/*
	show login form
*/
	public function testLogin()
	{
        $client = static::createClient();
        $crawler = $client->request('GET', '/login');
        $this->assertTrue($crawler->filter('html:contains("Authentication")')->count() > 0);	
	}
		
}
